/***************************************************************************
                         cbase32.h  -  description
                             -------------------
    begin                : Sat Apr 24 2004
    copyright            : (C) 2004 by Mathias Küster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CBASE32_H
#define CBASE32_H

/**
  *@author Mathias Küster
  *
  * Base32 encoder and decoder.
  */

#include <dclib/dcos.h>

class CByteArray;
class CString;

class CBase32 {

public:
	/** */
	static void Encode( CString * dst, CByteArray * src );
	/** */
	static unsigned long Decode( CByteArray * dst, CString * src );

protected:
	/** */
	CBase32();
	/** */
	~CBase32();

private:
	/** */
	static const signed char Index_32[256];
	/** */
	static const char B32Chars[33];
};

#endif
