/* Automatically generated file do not edit. */
/* It's useless on it's own and is included */
/* at the correct place in ccasefolder.cpp */
case 0x00041:
	folded[outpos] = 0x00061;
	++outpos;
	break;
case 0x00042:
	folded[outpos] = 0x00062;
	++outpos;
	break;
case 0x00043:
	folded[outpos] = 0x00063;
	++outpos;
	break;
case 0x00044:
	folded[outpos] = 0x00064;
	++outpos;
	break;
case 0x00045:
	folded[outpos] = 0x00065;
	++outpos;
	break;
case 0x00046:
	folded[outpos] = 0x00066;
	++outpos;
	break;
case 0x00047:
	folded[outpos] = 0x00067;
	++outpos;
	break;
case 0x00048:
	folded[outpos] = 0x00068;
	++outpos;
	break;
case 0x00049:
	folded[outpos] = 0x00069;
	++outpos;
	break;
case 0x0004A:
	folded[outpos] = 0x0006A;
	++outpos;
	break;
case 0x0004B:
	folded[outpos] = 0x0006B;
	++outpos;
	break;
case 0x0004C:
	folded[outpos] = 0x0006C;
	++outpos;
	break;
case 0x0004D:
	folded[outpos] = 0x0006D;
	++outpos;
	break;
case 0x0004E:
	folded[outpos] = 0x0006E;
	++outpos;
	break;
case 0x0004F:
	folded[outpos] = 0x0006F;
	++outpos;
	break;
case 0x00050:
	folded[outpos] = 0x00070;
	++outpos;
	break;
case 0x00051:
	folded[outpos] = 0x00071;
	++outpos;
	break;
case 0x00052:
	folded[outpos] = 0x00072;
	++outpos;
	break;
case 0x00053:
	folded[outpos] = 0x00073;
	++outpos;
	break;
case 0x00054:
	folded[outpos] = 0x00074;
	++outpos;
	break;
case 0x00055:
	folded[outpos] = 0x00075;
	++outpos;
	break;
case 0x00056:
	folded[outpos] = 0x00076;
	++outpos;
	break;
case 0x00057:
	folded[outpos] = 0x00077;
	++outpos;
	break;
case 0x00058:
	folded[outpos] = 0x00078;
	++outpos;
	break;
case 0x00059:
	folded[outpos] = 0x00079;
	++outpos;
	break;
case 0x0005A:
	folded[outpos] = 0x0007A;
	++outpos;
	break;
case 0x000B5:
	folded[outpos] = 0x003BC;
	++outpos;
	break;
case 0x000C0:
	folded[outpos] = 0x000E0;
	++outpos;
	break;
case 0x000C1:
	folded[outpos] = 0x000E1;
	++outpos;
	break;
case 0x000C2:
	folded[outpos] = 0x000E2;
	++outpos;
	break;
case 0x000C3:
	folded[outpos] = 0x000E3;
	++outpos;
	break;
case 0x000C4:
	folded[outpos] = 0x000E4;
	++outpos;
	break;
case 0x000C5:
	folded[outpos] = 0x000E5;
	++outpos;
	break;
case 0x000C6:
	folded[outpos] = 0x000E6;
	++outpos;
	break;
case 0x000C7:
	folded[outpos] = 0x000E7;
	++outpos;
	break;
case 0x000C8:
	folded[outpos] = 0x000E8;
	++outpos;
	break;
case 0x000C9:
	folded[outpos] = 0x000E9;
	++outpos;
	break;
case 0x000CA:
	folded[outpos] = 0x000EA;
	++outpos;
	break;
case 0x000CB:
	folded[outpos] = 0x000EB;
	++outpos;
	break;
case 0x000CC:
	folded[outpos] = 0x000EC;
	++outpos;
	break;
case 0x000CD:
	folded[outpos] = 0x000ED;
	++outpos;
	break;
case 0x000CE:
	folded[outpos] = 0x000EE;
	++outpos;
	break;
case 0x000CF:
	folded[outpos] = 0x000EF;
	++outpos;
	break;
case 0x000D0:
	folded[outpos] = 0x000F0;
	++outpos;
	break;
case 0x000D1:
	folded[outpos] = 0x000F1;
	++outpos;
	break;
case 0x000D2:
	folded[outpos] = 0x000F2;
	++outpos;
	break;
case 0x000D3:
	folded[outpos] = 0x000F3;
	++outpos;
	break;
case 0x000D4:
	folded[outpos] = 0x000F4;
	++outpos;
	break;
case 0x000D5:
	folded[outpos] = 0x000F5;
	++outpos;
	break;
case 0x000D6:
	folded[outpos] = 0x000F6;
	++outpos;
	break;
case 0x000D8:
	folded[outpos] = 0x000F8;
	++outpos;
	break;
case 0x000D9:
	folded[outpos] = 0x000F9;
	++outpos;
	break;
case 0x000DA:
	folded[outpos] = 0x000FA;
	++outpos;
	break;
case 0x000DB:
	folded[outpos] = 0x000FB;
	++outpos;
	break;
case 0x000DC:
	folded[outpos] = 0x000FC;
	++outpos;
	break;
case 0x000DD:
	folded[outpos] = 0x000FD;
	++outpos;
	break;
case 0x000DE:
	folded[outpos] = 0x000FE;
	++outpos;
	break;
case 0x000DF:
	folded[outpos] = 0x00073;
	++outpos;
	folded[outpos] = 0x00073;
	++outpos;
	break;
case 0x00100:
	folded[outpos] = 0x00101;
	++outpos;
	break;
case 0x00102:
	folded[outpos] = 0x00103;
	++outpos;
	break;
case 0x00104:
	folded[outpos] = 0x00105;
	++outpos;
	break;
case 0x00106:
	folded[outpos] = 0x00107;
	++outpos;
	break;
case 0x00108:
	folded[outpos] = 0x00109;
	++outpos;
	break;
case 0x0010A:
	folded[outpos] = 0x0010B;
	++outpos;
	break;
case 0x0010C:
	folded[outpos] = 0x0010D;
	++outpos;
	break;
case 0x0010E:
	folded[outpos] = 0x0010F;
	++outpos;
	break;
case 0x00110:
	folded[outpos] = 0x00111;
	++outpos;
	break;
case 0x00112:
	folded[outpos] = 0x00113;
	++outpos;
	break;
case 0x00114:
	folded[outpos] = 0x00115;
	++outpos;
	break;
case 0x00116:
	folded[outpos] = 0x00117;
	++outpos;
	break;
case 0x00118:
	folded[outpos] = 0x00119;
	++outpos;
	break;
case 0x0011A:
	folded[outpos] = 0x0011B;
	++outpos;
	break;
case 0x0011C:
	folded[outpos] = 0x0011D;
	++outpos;
	break;
case 0x0011E:
	folded[outpos] = 0x0011F;
	++outpos;
	break;
case 0x00120:
	folded[outpos] = 0x00121;
	++outpos;
	break;
case 0x00122:
	folded[outpos] = 0x00123;
	++outpos;
	break;
case 0x00124:
	folded[outpos] = 0x00125;
	++outpos;
	break;
case 0x00126:
	folded[outpos] = 0x00127;
	++outpos;
	break;
case 0x00128:
	folded[outpos] = 0x00129;
	++outpos;
	break;
case 0x0012A:
	folded[outpos] = 0x0012B;
	++outpos;
	break;
case 0x0012C:
	folded[outpos] = 0x0012D;
	++outpos;
	break;
case 0x0012E:
	folded[outpos] = 0x0012F;
	++outpos;
	break;
case 0x00130:
	folded[outpos] = 0x00069;
	++outpos;
	folded[outpos] = 0x00307;
	++outpos;
	break;
case 0x00132:
	folded[outpos] = 0x00133;
	++outpos;
	break;
case 0x00134:
	folded[outpos] = 0x00135;
	++outpos;
	break;
case 0x00136:
	folded[outpos] = 0x00137;
	++outpos;
	break;
case 0x00139:
	folded[outpos] = 0x0013A;
	++outpos;
	break;
case 0x0013B:
	folded[outpos] = 0x0013C;
	++outpos;
	break;
case 0x0013D:
	folded[outpos] = 0x0013E;
	++outpos;
	break;
case 0x0013F:
	folded[outpos] = 0x00140;
	++outpos;
	break;
case 0x00141:
	folded[outpos] = 0x00142;
	++outpos;
	break;
case 0x00143:
	folded[outpos] = 0x00144;
	++outpos;
	break;
case 0x00145:
	folded[outpos] = 0x00146;
	++outpos;
	break;
case 0x00147:
	folded[outpos] = 0x00148;
	++outpos;
	break;
case 0x00149:
	folded[outpos] = 0x002BC;
	++outpos;
	folded[outpos] = 0x0006E;
	++outpos;
	break;
case 0x0014A:
	folded[outpos] = 0x0014B;
	++outpos;
	break;
case 0x0014C:
	folded[outpos] = 0x0014D;
	++outpos;
	break;
case 0x0014E:
	folded[outpos] = 0x0014F;
	++outpos;
	break;
case 0x00150:
	folded[outpos] = 0x00151;
	++outpos;
	break;
case 0x00152:
	folded[outpos] = 0x00153;
	++outpos;
	break;
case 0x00154:
	folded[outpos] = 0x00155;
	++outpos;
	break;
case 0x00156:
	folded[outpos] = 0x00157;
	++outpos;
	break;
case 0x00158:
	folded[outpos] = 0x00159;
	++outpos;
	break;
case 0x0015A:
	folded[outpos] = 0x0015B;
	++outpos;
	break;
case 0x0015C:
	folded[outpos] = 0x0015D;
	++outpos;
	break;
case 0x0015E:
	folded[outpos] = 0x0015F;
	++outpos;
	break;
case 0x00160:
	folded[outpos] = 0x00161;
	++outpos;
	break;
case 0x00162:
	folded[outpos] = 0x00163;
	++outpos;
	break;
case 0x00164:
	folded[outpos] = 0x00165;
	++outpos;
	break;
case 0x00166:
	folded[outpos] = 0x00167;
	++outpos;
	break;
case 0x00168:
	folded[outpos] = 0x00169;
	++outpos;
	break;
case 0x0016A:
	folded[outpos] = 0x0016B;
	++outpos;
	break;
case 0x0016C:
	folded[outpos] = 0x0016D;
	++outpos;
	break;
case 0x0016E:
	folded[outpos] = 0x0016F;
	++outpos;
	break;
case 0x00170:
	folded[outpos] = 0x00171;
	++outpos;
	break;
case 0x00172:
	folded[outpos] = 0x00173;
	++outpos;
	break;
case 0x00174:
	folded[outpos] = 0x00175;
	++outpos;
	break;
case 0x00176:
	folded[outpos] = 0x00177;
	++outpos;
	break;
case 0x00178:
	folded[outpos] = 0x000FF;
	++outpos;
	break;
case 0x00179:
	folded[outpos] = 0x0017A;
	++outpos;
	break;
case 0x0017B:
	folded[outpos] = 0x0017C;
	++outpos;
	break;
case 0x0017D:
	folded[outpos] = 0x0017E;
	++outpos;
	break;
case 0x0017F:
	folded[outpos] = 0x00073;
	++outpos;
	break;
case 0x00181:
	folded[outpos] = 0x00253;
	++outpos;
	break;
case 0x00182:
	folded[outpos] = 0x00183;
	++outpos;
	break;
case 0x00184:
	folded[outpos] = 0x00185;
	++outpos;
	break;
case 0x00186:
	folded[outpos] = 0x00254;
	++outpos;
	break;
case 0x00187:
	folded[outpos] = 0x00188;
	++outpos;
	break;
case 0x00189:
	folded[outpos] = 0x00256;
	++outpos;
	break;
case 0x0018A:
	folded[outpos] = 0x00257;
	++outpos;
	break;
case 0x0018B:
	folded[outpos] = 0x0018C;
	++outpos;
	break;
case 0x0018E:
	folded[outpos] = 0x001DD;
	++outpos;
	break;
case 0x0018F:
	folded[outpos] = 0x00259;
	++outpos;
	break;
case 0x00190:
	folded[outpos] = 0x0025B;
	++outpos;
	break;
case 0x00191:
	folded[outpos] = 0x00192;
	++outpos;
	break;
case 0x00193:
	folded[outpos] = 0x00260;
	++outpos;
	break;
case 0x00194:
	folded[outpos] = 0x00263;
	++outpos;
	break;
case 0x00196:
	folded[outpos] = 0x00269;
	++outpos;
	break;
case 0x00197:
	folded[outpos] = 0x00268;
	++outpos;
	break;
case 0x00198:
	folded[outpos] = 0x00199;
	++outpos;
	break;
case 0x0019C:
	folded[outpos] = 0x0026F;
	++outpos;
	break;
case 0x0019D:
	folded[outpos] = 0x00272;
	++outpos;
	break;
case 0x0019F:
	folded[outpos] = 0x00275;
	++outpos;
	break;
case 0x001A0:
	folded[outpos] = 0x001A1;
	++outpos;
	break;
case 0x001A2:
	folded[outpos] = 0x001A3;
	++outpos;
	break;
case 0x001A4:
	folded[outpos] = 0x001A5;
	++outpos;
	break;
case 0x001A6:
	folded[outpos] = 0x00280;
	++outpos;
	break;
case 0x001A7:
	folded[outpos] = 0x001A8;
	++outpos;
	break;
case 0x001A9:
	folded[outpos] = 0x00283;
	++outpos;
	break;
case 0x001AC:
	folded[outpos] = 0x001AD;
	++outpos;
	break;
case 0x001AE:
	folded[outpos] = 0x00288;
	++outpos;
	break;
case 0x001AF:
	folded[outpos] = 0x001B0;
	++outpos;
	break;
case 0x001B1:
	folded[outpos] = 0x0028A;
	++outpos;
	break;
case 0x001B2:
	folded[outpos] = 0x0028B;
	++outpos;
	break;
case 0x001B3:
	folded[outpos] = 0x001B4;
	++outpos;
	break;
case 0x001B5:
	folded[outpos] = 0x001B6;
	++outpos;
	break;
case 0x001B7:
	folded[outpos] = 0x00292;
	++outpos;
	break;
case 0x001B8:
	folded[outpos] = 0x001B9;
	++outpos;
	break;
case 0x001BC:
	folded[outpos] = 0x001BD;
	++outpos;
	break;
case 0x001C4:
	folded[outpos] = 0x001C6;
	++outpos;
	break;
case 0x001C5:
	folded[outpos] = 0x001C6;
	++outpos;
	break;
case 0x001C7:
	folded[outpos] = 0x001C9;
	++outpos;
	break;
case 0x001C8:
	folded[outpos] = 0x001C9;
	++outpos;
	break;
case 0x001CA:
	folded[outpos] = 0x001CC;
	++outpos;
	break;
case 0x001CB:
	folded[outpos] = 0x001CC;
	++outpos;
	break;
case 0x001CD:
	folded[outpos] = 0x001CE;
	++outpos;
	break;
case 0x001CF:
	folded[outpos] = 0x001D0;
	++outpos;
	break;
case 0x001D1:
	folded[outpos] = 0x001D2;
	++outpos;
	break;
case 0x001D3:
	folded[outpos] = 0x001D4;
	++outpos;
	break;
case 0x001D5:
	folded[outpos] = 0x001D6;
	++outpos;
	break;
case 0x001D7:
	folded[outpos] = 0x001D8;
	++outpos;
	break;
case 0x001D9:
	folded[outpos] = 0x001DA;
	++outpos;
	break;
case 0x001DB:
	folded[outpos] = 0x001DC;
	++outpos;
	break;
case 0x001DE:
	folded[outpos] = 0x001DF;
	++outpos;
	break;
case 0x001E0:
	folded[outpos] = 0x001E1;
	++outpos;
	break;
case 0x001E2:
	folded[outpos] = 0x001E3;
	++outpos;
	break;
case 0x001E4:
	folded[outpos] = 0x001E5;
	++outpos;
	break;
case 0x001E6:
	folded[outpos] = 0x001E7;
	++outpos;
	break;
case 0x001E8:
	folded[outpos] = 0x001E9;
	++outpos;
	break;
case 0x001EA:
	folded[outpos] = 0x001EB;
	++outpos;
	break;
case 0x001EC:
	folded[outpos] = 0x001ED;
	++outpos;
	break;
case 0x001EE:
	folded[outpos] = 0x001EF;
	++outpos;
	break;
case 0x001F0:
	folded[outpos] = 0x0006A;
	++outpos;
	folded[outpos] = 0x0030C;
	++outpos;
	break;
case 0x001F1:
	folded[outpos] = 0x001F3;
	++outpos;
	break;
case 0x001F2:
	folded[outpos] = 0x001F3;
	++outpos;
	break;
case 0x001F4:
	folded[outpos] = 0x001F5;
	++outpos;
	break;
case 0x001F6:
	folded[outpos] = 0x00195;
	++outpos;
	break;
case 0x001F7:
	folded[outpos] = 0x001BF;
	++outpos;
	break;
case 0x001F8:
	folded[outpos] = 0x001F9;
	++outpos;
	break;
case 0x001FA:
	folded[outpos] = 0x001FB;
	++outpos;
	break;
case 0x001FC:
	folded[outpos] = 0x001FD;
	++outpos;
	break;
case 0x001FE:
	folded[outpos] = 0x001FF;
	++outpos;
	break;
case 0x00200:
	folded[outpos] = 0x00201;
	++outpos;
	break;
case 0x00202:
	folded[outpos] = 0x00203;
	++outpos;
	break;
case 0x00204:
	folded[outpos] = 0x00205;
	++outpos;
	break;
case 0x00206:
	folded[outpos] = 0x00207;
	++outpos;
	break;
case 0x00208:
	folded[outpos] = 0x00209;
	++outpos;
	break;
case 0x0020A:
	folded[outpos] = 0x0020B;
	++outpos;
	break;
case 0x0020C:
	folded[outpos] = 0x0020D;
	++outpos;
	break;
case 0x0020E:
	folded[outpos] = 0x0020F;
	++outpos;
	break;
case 0x00210:
	folded[outpos] = 0x00211;
	++outpos;
	break;
case 0x00212:
	folded[outpos] = 0x00213;
	++outpos;
	break;
case 0x00214:
	folded[outpos] = 0x00215;
	++outpos;
	break;
case 0x00216:
	folded[outpos] = 0x00217;
	++outpos;
	break;
case 0x00218:
	folded[outpos] = 0x00219;
	++outpos;
	break;
case 0x0021A:
	folded[outpos] = 0x0021B;
	++outpos;
	break;
case 0x0021C:
	folded[outpos] = 0x0021D;
	++outpos;
	break;
case 0x0021E:
	folded[outpos] = 0x0021F;
	++outpos;
	break;
case 0x00220:
	folded[outpos] = 0x0019E;
	++outpos;
	break;
case 0x00222:
	folded[outpos] = 0x00223;
	++outpos;
	break;
case 0x00224:
	folded[outpos] = 0x00225;
	++outpos;
	break;
case 0x00226:
	folded[outpos] = 0x00227;
	++outpos;
	break;
case 0x00228:
	folded[outpos] = 0x00229;
	++outpos;
	break;
case 0x0022A:
	folded[outpos] = 0x0022B;
	++outpos;
	break;
case 0x0022C:
	folded[outpos] = 0x0022D;
	++outpos;
	break;
case 0x0022E:
	folded[outpos] = 0x0022F;
	++outpos;
	break;
case 0x00230:
	folded[outpos] = 0x00231;
	++outpos;
	break;
case 0x00232:
	folded[outpos] = 0x00233;
	++outpos;
	break;
case 0x0023A:
	folded[outpos] = 0x02C65;
	++outpos;
	break;
case 0x0023B:
	folded[outpos] = 0x0023C;
	++outpos;
	break;
case 0x0023D:
	folded[outpos] = 0x0019A;
	++outpos;
	break;
case 0x0023E:
	folded[outpos] = 0x02C66;
	++outpos;
	break;
case 0x00241:
	folded[outpos] = 0x00242;
	++outpos;
	break;
case 0x00243:
	folded[outpos] = 0x00180;
	++outpos;
	break;
case 0x00244:
	folded[outpos] = 0x00289;
	++outpos;
	break;
case 0x00245:
	folded[outpos] = 0x0028C;
	++outpos;
	break;
case 0x00246:
	folded[outpos] = 0x00247;
	++outpos;
	break;
case 0x00248:
	folded[outpos] = 0x00249;
	++outpos;
	break;
case 0x0024A:
	folded[outpos] = 0x0024B;
	++outpos;
	break;
case 0x0024C:
	folded[outpos] = 0x0024D;
	++outpos;
	break;
case 0x0024E:
	folded[outpos] = 0x0024F;
	++outpos;
	break;
case 0x00345:
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x00370:
	folded[outpos] = 0x00371;
	++outpos;
	break;
case 0x00372:
	folded[outpos] = 0x00373;
	++outpos;
	break;
case 0x00376:
	folded[outpos] = 0x00377;
	++outpos;
	break;
case 0x00386:
	folded[outpos] = 0x003AC;
	++outpos;
	break;
case 0x00388:
	folded[outpos] = 0x003AD;
	++outpos;
	break;
case 0x00389:
	folded[outpos] = 0x003AE;
	++outpos;
	break;
case 0x0038A:
	folded[outpos] = 0x003AF;
	++outpos;
	break;
case 0x0038C:
	folded[outpos] = 0x003CC;
	++outpos;
	break;
case 0x0038E:
	folded[outpos] = 0x003CD;
	++outpos;
	break;
case 0x0038F:
	folded[outpos] = 0x003CE;
	++outpos;
	break;
case 0x00390:
	folded[outpos] = 0x003B9;
	++outpos;
	folded[outpos] = 0x00308;
	++outpos;
	folded[outpos] = 0x00301;
	++outpos;
	break;
case 0x00391:
	folded[outpos] = 0x003B1;
	++outpos;
	break;
case 0x00392:
	folded[outpos] = 0x003B2;
	++outpos;
	break;
case 0x00393:
	folded[outpos] = 0x003B3;
	++outpos;
	break;
case 0x00394:
	folded[outpos] = 0x003B4;
	++outpos;
	break;
case 0x00395:
	folded[outpos] = 0x003B5;
	++outpos;
	break;
case 0x00396:
	folded[outpos] = 0x003B6;
	++outpos;
	break;
case 0x00397:
	folded[outpos] = 0x003B7;
	++outpos;
	break;
case 0x00398:
	folded[outpos] = 0x003B8;
	++outpos;
	break;
case 0x00399:
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x0039A:
	folded[outpos] = 0x003BA;
	++outpos;
	break;
case 0x0039B:
	folded[outpos] = 0x003BB;
	++outpos;
	break;
case 0x0039C:
	folded[outpos] = 0x003BC;
	++outpos;
	break;
case 0x0039D:
	folded[outpos] = 0x003BD;
	++outpos;
	break;
case 0x0039E:
	folded[outpos] = 0x003BE;
	++outpos;
	break;
case 0x0039F:
	folded[outpos] = 0x003BF;
	++outpos;
	break;
case 0x003A0:
	folded[outpos] = 0x003C0;
	++outpos;
	break;
case 0x003A1:
	folded[outpos] = 0x003C1;
	++outpos;
	break;
case 0x003A3:
	folded[outpos] = 0x003C3;
	++outpos;
	break;
case 0x003A4:
	folded[outpos] = 0x003C4;
	++outpos;
	break;
case 0x003A5:
	folded[outpos] = 0x003C5;
	++outpos;
	break;
case 0x003A6:
	folded[outpos] = 0x003C6;
	++outpos;
	break;
case 0x003A7:
	folded[outpos] = 0x003C7;
	++outpos;
	break;
case 0x003A8:
	folded[outpos] = 0x003C8;
	++outpos;
	break;
case 0x003A9:
	folded[outpos] = 0x003C9;
	++outpos;
	break;
case 0x003AA:
	folded[outpos] = 0x003CA;
	++outpos;
	break;
case 0x003AB:
	folded[outpos] = 0x003CB;
	++outpos;
	break;
case 0x003B0:
	folded[outpos] = 0x003C5;
	++outpos;
	folded[outpos] = 0x00308;
	++outpos;
	folded[outpos] = 0x00301;
	++outpos;
	break;
case 0x003C2:
	folded[outpos] = 0x003C3;
	++outpos;
	break;
case 0x003CF:
	folded[outpos] = 0x003D7;
	++outpos;
	break;
case 0x003D0:
	folded[outpos] = 0x003B2;
	++outpos;
	break;
case 0x003D1:
	folded[outpos] = 0x003B8;
	++outpos;
	break;
case 0x003D5:
	folded[outpos] = 0x003C6;
	++outpos;
	break;
case 0x003D6:
	folded[outpos] = 0x003C0;
	++outpos;
	break;
case 0x003D8:
	folded[outpos] = 0x003D9;
	++outpos;
	break;
case 0x003DA:
	folded[outpos] = 0x003DB;
	++outpos;
	break;
case 0x003DC:
	folded[outpos] = 0x003DD;
	++outpos;
	break;
case 0x003DE:
	folded[outpos] = 0x003DF;
	++outpos;
	break;
case 0x003E0:
	folded[outpos] = 0x003E1;
	++outpos;
	break;
case 0x003E2:
	folded[outpos] = 0x003E3;
	++outpos;
	break;
case 0x003E4:
	folded[outpos] = 0x003E5;
	++outpos;
	break;
case 0x003E6:
	folded[outpos] = 0x003E7;
	++outpos;
	break;
case 0x003E8:
	folded[outpos] = 0x003E9;
	++outpos;
	break;
case 0x003EA:
	folded[outpos] = 0x003EB;
	++outpos;
	break;
case 0x003EC:
	folded[outpos] = 0x003ED;
	++outpos;
	break;
case 0x003EE:
	folded[outpos] = 0x003EF;
	++outpos;
	break;
case 0x003F0:
	folded[outpos] = 0x003BA;
	++outpos;
	break;
case 0x003F1:
	folded[outpos] = 0x003C1;
	++outpos;
	break;
case 0x003F4:
	folded[outpos] = 0x003B8;
	++outpos;
	break;
case 0x003F5:
	folded[outpos] = 0x003B5;
	++outpos;
	break;
case 0x003F7:
	folded[outpos] = 0x003F8;
	++outpos;
	break;
case 0x003F9:
	folded[outpos] = 0x003F2;
	++outpos;
	break;
case 0x003FA:
	folded[outpos] = 0x003FB;
	++outpos;
	break;
case 0x003FD:
	folded[outpos] = 0x0037B;
	++outpos;
	break;
case 0x003FE:
	folded[outpos] = 0x0037C;
	++outpos;
	break;
case 0x003FF:
	folded[outpos] = 0x0037D;
	++outpos;
	break;
case 0x00400:
	folded[outpos] = 0x00450;
	++outpos;
	break;
case 0x00401:
	folded[outpos] = 0x00451;
	++outpos;
	break;
case 0x00402:
	folded[outpos] = 0x00452;
	++outpos;
	break;
case 0x00403:
	folded[outpos] = 0x00453;
	++outpos;
	break;
case 0x00404:
	folded[outpos] = 0x00454;
	++outpos;
	break;
case 0x00405:
	folded[outpos] = 0x00455;
	++outpos;
	break;
case 0x00406:
	folded[outpos] = 0x00456;
	++outpos;
	break;
case 0x00407:
	folded[outpos] = 0x00457;
	++outpos;
	break;
case 0x00408:
	folded[outpos] = 0x00458;
	++outpos;
	break;
case 0x00409:
	folded[outpos] = 0x00459;
	++outpos;
	break;
case 0x0040A:
	folded[outpos] = 0x0045A;
	++outpos;
	break;
case 0x0040B:
	folded[outpos] = 0x0045B;
	++outpos;
	break;
case 0x0040C:
	folded[outpos] = 0x0045C;
	++outpos;
	break;
case 0x0040D:
	folded[outpos] = 0x0045D;
	++outpos;
	break;
case 0x0040E:
	folded[outpos] = 0x0045E;
	++outpos;
	break;
case 0x0040F:
	folded[outpos] = 0x0045F;
	++outpos;
	break;
case 0x00410:
	folded[outpos] = 0x00430;
	++outpos;
	break;
case 0x00411:
	folded[outpos] = 0x00431;
	++outpos;
	break;
case 0x00412:
	folded[outpos] = 0x00432;
	++outpos;
	break;
case 0x00413:
	folded[outpos] = 0x00433;
	++outpos;
	break;
case 0x00414:
	folded[outpos] = 0x00434;
	++outpos;
	break;
case 0x00415:
	folded[outpos] = 0x00435;
	++outpos;
	break;
case 0x00416:
	folded[outpos] = 0x00436;
	++outpos;
	break;
case 0x00417:
	folded[outpos] = 0x00437;
	++outpos;
	break;
case 0x00418:
	folded[outpos] = 0x00438;
	++outpos;
	break;
case 0x00419:
	folded[outpos] = 0x00439;
	++outpos;
	break;
case 0x0041A:
	folded[outpos] = 0x0043A;
	++outpos;
	break;
case 0x0041B:
	folded[outpos] = 0x0043B;
	++outpos;
	break;
case 0x0041C:
	folded[outpos] = 0x0043C;
	++outpos;
	break;
case 0x0041D:
	folded[outpos] = 0x0043D;
	++outpos;
	break;
case 0x0041E:
	folded[outpos] = 0x0043E;
	++outpos;
	break;
case 0x0041F:
	folded[outpos] = 0x0043F;
	++outpos;
	break;
case 0x00420:
	folded[outpos] = 0x00440;
	++outpos;
	break;
case 0x00421:
	folded[outpos] = 0x00441;
	++outpos;
	break;
case 0x00422:
	folded[outpos] = 0x00442;
	++outpos;
	break;
case 0x00423:
	folded[outpos] = 0x00443;
	++outpos;
	break;
case 0x00424:
	folded[outpos] = 0x00444;
	++outpos;
	break;
case 0x00425:
	folded[outpos] = 0x00445;
	++outpos;
	break;
case 0x00426:
	folded[outpos] = 0x00446;
	++outpos;
	break;
case 0x00427:
	folded[outpos] = 0x00447;
	++outpos;
	break;
case 0x00428:
	folded[outpos] = 0x00448;
	++outpos;
	break;
case 0x00429:
	folded[outpos] = 0x00449;
	++outpos;
	break;
case 0x0042A:
	folded[outpos] = 0x0044A;
	++outpos;
	break;
case 0x0042B:
	folded[outpos] = 0x0044B;
	++outpos;
	break;
case 0x0042C:
	folded[outpos] = 0x0044C;
	++outpos;
	break;
case 0x0042D:
	folded[outpos] = 0x0044D;
	++outpos;
	break;
case 0x0042E:
	folded[outpos] = 0x0044E;
	++outpos;
	break;
case 0x0042F:
	folded[outpos] = 0x0044F;
	++outpos;
	break;
case 0x00460:
	folded[outpos] = 0x00461;
	++outpos;
	break;
case 0x00462:
	folded[outpos] = 0x00463;
	++outpos;
	break;
case 0x00464:
	folded[outpos] = 0x00465;
	++outpos;
	break;
case 0x00466:
	folded[outpos] = 0x00467;
	++outpos;
	break;
case 0x00468:
	folded[outpos] = 0x00469;
	++outpos;
	break;
case 0x0046A:
	folded[outpos] = 0x0046B;
	++outpos;
	break;
case 0x0046C:
	folded[outpos] = 0x0046D;
	++outpos;
	break;
case 0x0046E:
	folded[outpos] = 0x0046F;
	++outpos;
	break;
case 0x00470:
	folded[outpos] = 0x00471;
	++outpos;
	break;
case 0x00472:
	folded[outpos] = 0x00473;
	++outpos;
	break;
case 0x00474:
	folded[outpos] = 0x00475;
	++outpos;
	break;
case 0x00476:
	folded[outpos] = 0x00477;
	++outpos;
	break;
case 0x00478:
	folded[outpos] = 0x00479;
	++outpos;
	break;
case 0x0047A:
	folded[outpos] = 0x0047B;
	++outpos;
	break;
case 0x0047C:
	folded[outpos] = 0x0047D;
	++outpos;
	break;
case 0x0047E:
	folded[outpos] = 0x0047F;
	++outpos;
	break;
case 0x00480:
	folded[outpos] = 0x00481;
	++outpos;
	break;
case 0x0048A:
	folded[outpos] = 0x0048B;
	++outpos;
	break;
case 0x0048C:
	folded[outpos] = 0x0048D;
	++outpos;
	break;
case 0x0048E:
	folded[outpos] = 0x0048F;
	++outpos;
	break;
case 0x00490:
	folded[outpos] = 0x00491;
	++outpos;
	break;
case 0x00492:
	folded[outpos] = 0x00493;
	++outpos;
	break;
case 0x00494:
	folded[outpos] = 0x00495;
	++outpos;
	break;
case 0x00496:
	folded[outpos] = 0x00497;
	++outpos;
	break;
case 0x00498:
	folded[outpos] = 0x00499;
	++outpos;
	break;
case 0x0049A:
	folded[outpos] = 0x0049B;
	++outpos;
	break;
case 0x0049C:
	folded[outpos] = 0x0049D;
	++outpos;
	break;
case 0x0049E:
	folded[outpos] = 0x0049F;
	++outpos;
	break;
case 0x004A0:
	folded[outpos] = 0x004A1;
	++outpos;
	break;
case 0x004A2:
	folded[outpos] = 0x004A3;
	++outpos;
	break;
case 0x004A4:
	folded[outpos] = 0x004A5;
	++outpos;
	break;
case 0x004A6:
	folded[outpos] = 0x004A7;
	++outpos;
	break;
case 0x004A8:
	folded[outpos] = 0x004A9;
	++outpos;
	break;
case 0x004AA:
	folded[outpos] = 0x004AB;
	++outpos;
	break;
case 0x004AC:
	folded[outpos] = 0x004AD;
	++outpos;
	break;
case 0x004AE:
	folded[outpos] = 0x004AF;
	++outpos;
	break;
case 0x004B0:
	folded[outpos] = 0x004B1;
	++outpos;
	break;
case 0x004B2:
	folded[outpos] = 0x004B3;
	++outpos;
	break;
case 0x004B4:
	folded[outpos] = 0x004B5;
	++outpos;
	break;
case 0x004B6:
	folded[outpos] = 0x004B7;
	++outpos;
	break;
case 0x004B8:
	folded[outpos] = 0x004B9;
	++outpos;
	break;
case 0x004BA:
	folded[outpos] = 0x004BB;
	++outpos;
	break;
case 0x004BC:
	folded[outpos] = 0x004BD;
	++outpos;
	break;
case 0x004BE:
	folded[outpos] = 0x004BF;
	++outpos;
	break;
case 0x004C0:
	folded[outpos] = 0x004CF;
	++outpos;
	break;
case 0x004C1:
	folded[outpos] = 0x004C2;
	++outpos;
	break;
case 0x004C3:
	folded[outpos] = 0x004C4;
	++outpos;
	break;
case 0x004C5:
	folded[outpos] = 0x004C6;
	++outpos;
	break;
case 0x004C7:
	folded[outpos] = 0x004C8;
	++outpos;
	break;
case 0x004C9:
	folded[outpos] = 0x004CA;
	++outpos;
	break;
case 0x004CB:
	folded[outpos] = 0x004CC;
	++outpos;
	break;
case 0x004CD:
	folded[outpos] = 0x004CE;
	++outpos;
	break;
case 0x004D0:
	folded[outpos] = 0x004D1;
	++outpos;
	break;
case 0x004D2:
	folded[outpos] = 0x004D3;
	++outpos;
	break;
case 0x004D4:
	folded[outpos] = 0x004D5;
	++outpos;
	break;
case 0x004D6:
	folded[outpos] = 0x004D7;
	++outpos;
	break;
case 0x004D8:
	folded[outpos] = 0x004D9;
	++outpos;
	break;
case 0x004DA:
	folded[outpos] = 0x004DB;
	++outpos;
	break;
case 0x004DC:
	folded[outpos] = 0x004DD;
	++outpos;
	break;
case 0x004DE:
	folded[outpos] = 0x004DF;
	++outpos;
	break;
case 0x004E0:
	folded[outpos] = 0x004E1;
	++outpos;
	break;
case 0x004E2:
	folded[outpos] = 0x004E3;
	++outpos;
	break;
case 0x004E4:
	folded[outpos] = 0x004E5;
	++outpos;
	break;
case 0x004E6:
	folded[outpos] = 0x004E7;
	++outpos;
	break;
case 0x004E8:
	folded[outpos] = 0x004E9;
	++outpos;
	break;
case 0x004EA:
	folded[outpos] = 0x004EB;
	++outpos;
	break;
case 0x004EC:
	folded[outpos] = 0x004ED;
	++outpos;
	break;
case 0x004EE:
	folded[outpos] = 0x004EF;
	++outpos;
	break;
case 0x004F0:
	folded[outpos] = 0x004F1;
	++outpos;
	break;
case 0x004F2:
	folded[outpos] = 0x004F3;
	++outpos;
	break;
case 0x004F4:
	folded[outpos] = 0x004F5;
	++outpos;
	break;
case 0x004F6:
	folded[outpos] = 0x004F7;
	++outpos;
	break;
case 0x004F8:
	folded[outpos] = 0x004F9;
	++outpos;
	break;
case 0x004FA:
	folded[outpos] = 0x004FB;
	++outpos;
	break;
case 0x004FC:
	folded[outpos] = 0x004FD;
	++outpos;
	break;
case 0x004FE:
	folded[outpos] = 0x004FF;
	++outpos;
	break;
case 0x00500:
	folded[outpos] = 0x00501;
	++outpos;
	break;
case 0x00502:
	folded[outpos] = 0x00503;
	++outpos;
	break;
case 0x00504:
	folded[outpos] = 0x00505;
	++outpos;
	break;
case 0x00506:
	folded[outpos] = 0x00507;
	++outpos;
	break;
case 0x00508:
	folded[outpos] = 0x00509;
	++outpos;
	break;
case 0x0050A:
	folded[outpos] = 0x0050B;
	++outpos;
	break;
case 0x0050C:
	folded[outpos] = 0x0050D;
	++outpos;
	break;
case 0x0050E:
	folded[outpos] = 0x0050F;
	++outpos;
	break;
case 0x00510:
	folded[outpos] = 0x00511;
	++outpos;
	break;
case 0x00512:
	folded[outpos] = 0x00513;
	++outpos;
	break;
case 0x00514:
	folded[outpos] = 0x00515;
	++outpos;
	break;
case 0x00516:
	folded[outpos] = 0x00517;
	++outpos;
	break;
case 0x00518:
	folded[outpos] = 0x00519;
	++outpos;
	break;
case 0x0051A:
	folded[outpos] = 0x0051B;
	++outpos;
	break;
case 0x0051C:
	folded[outpos] = 0x0051D;
	++outpos;
	break;
case 0x0051E:
	folded[outpos] = 0x0051F;
	++outpos;
	break;
case 0x00520:
	folded[outpos] = 0x00521;
	++outpos;
	break;
case 0x00522:
	folded[outpos] = 0x00523;
	++outpos;
	break;
case 0x00531:
	folded[outpos] = 0x00561;
	++outpos;
	break;
case 0x00532:
	folded[outpos] = 0x00562;
	++outpos;
	break;
case 0x00533:
	folded[outpos] = 0x00563;
	++outpos;
	break;
case 0x00534:
	folded[outpos] = 0x00564;
	++outpos;
	break;
case 0x00535:
	folded[outpos] = 0x00565;
	++outpos;
	break;
case 0x00536:
	folded[outpos] = 0x00566;
	++outpos;
	break;
case 0x00537:
	folded[outpos] = 0x00567;
	++outpos;
	break;
case 0x00538:
	folded[outpos] = 0x00568;
	++outpos;
	break;
case 0x00539:
	folded[outpos] = 0x00569;
	++outpos;
	break;
case 0x0053A:
	folded[outpos] = 0x0056A;
	++outpos;
	break;
case 0x0053B:
	folded[outpos] = 0x0056B;
	++outpos;
	break;
case 0x0053C:
	folded[outpos] = 0x0056C;
	++outpos;
	break;
case 0x0053D:
	folded[outpos] = 0x0056D;
	++outpos;
	break;
case 0x0053E:
	folded[outpos] = 0x0056E;
	++outpos;
	break;
case 0x0053F:
	folded[outpos] = 0x0056F;
	++outpos;
	break;
case 0x00540:
	folded[outpos] = 0x00570;
	++outpos;
	break;
case 0x00541:
	folded[outpos] = 0x00571;
	++outpos;
	break;
case 0x00542:
	folded[outpos] = 0x00572;
	++outpos;
	break;
case 0x00543:
	folded[outpos] = 0x00573;
	++outpos;
	break;
case 0x00544:
	folded[outpos] = 0x00574;
	++outpos;
	break;
case 0x00545:
	folded[outpos] = 0x00575;
	++outpos;
	break;
case 0x00546:
	folded[outpos] = 0x00576;
	++outpos;
	break;
case 0x00547:
	folded[outpos] = 0x00577;
	++outpos;
	break;
case 0x00548:
	folded[outpos] = 0x00578;
	++outpos;
	break;
case 0x00549:
	folded[outpos] = 0x00579;
	++outpos;
	break;
case 0x0054A:
	folded[outpos] = 0x0057A;
	++outpos;
	break;
case 0x0054B:
	folded[outpos] = 0x0057B;
	++outpos;
	break;
case 0x0054C:
	folded[outpos] = 0x0057C;
	++outpos;
	break;
case 0x0054D:
	folded[outpos] = 0x0057D;
	++outpos;
	break;
case 0x0054E:
	folded[outpos] = 0x0057E;
	++outpos;
	break;
case 0x0054F:
	folded[outpos] = 0x0057F;
	++outpos;
	break;
case 0x00550:
	folded[outpos] = 0x00580;
	++outpos;
	break;
case 0x00551:
	folded[outpos] = 0x00581;
	++outpos;
	break;
case 0x00552:
	folded[outpos] = 0x00582;
	++outpos;
	break;
case 0x00553:
	folded[outpos] = 0x00583;
	++outpos;
	break;
case 0x00554:
	folded[outpos] = 0x00584;
	++outpos;
	break;
case 0x00555:
	folded[outpos] = 0x00585;
	++outpos;
	break;
case 0x00556:
	folded[outpos] = 0x00586;
	++outpos;
	break;
case 0x00587:
	folded[outpos] = 0x00565;
	++outpos;
	folded[outpos] = 0x00582;
	++outpos;
	break;
case 0x010A0:
	folded[outpos] = 0x02D00;
	++outpos;
	break;
case 0x010A1:
	folded[outpos] = 0x02D01;
	++outpos;
	break;
case 0x010A2:
	folded[outpos] = 0x02D02;
	++outpos;
	break;
case 0x010A3:
	folded[outpos] = 0x02D03;
	++outpos;
	break;
case 0x010A4:
	folded[outpos] = 0x02D04;
	++outpos;
	break;
case 0x010A5:
	folded[outpos] = 0x02D05;
	++outpos;
	break;
case 0x010A6:
	folded[outpos] = 0x02D06;
	++outpos;
	break;
case 0x010A7:
	folded[outpos] = 0x02D07;
	++outpos;
	break;
case 0x010A8:
	folded[outpos] = 0x02D08;
	++outpos;
	break;
case 0x010A9:
	folded[outpos] = 0x02D09;
	++outpos;
	break;
case 0x010AA:
	folded[outpos] = 0x02D0A;
	++outpos;
	break;
case 0x010AB:
	folded[outpos] = 0x02D0B;
	++outpos;
	break;
case 0x010AC:
	folded[outpos] = 0x02D0C;
	++outpos;
	break;
case 0x010AD:
	folded[outpos] = 0x02D0D;
	++outpos;
	break;
case 0x010AE:
	folded[outpos] = 0x02D0E;
	++outpos;
	break;
case 0x010AF:
	folded[outpos] = 0x02D0F;
	++outpos;
	break;
case 0x010B0:
	folded[outpos] = 0x02D10;
	++outpos;
	break;
case 0x010B1:
	folded[outpos] = 0x02D11;
	++outpos;
	break;
case 0x010B2:
	folded[outpos] = 0x02D12;
	++outpos;
	break;
case 0x010B3:
	folded[outpos] = 0x02D13;
	++outpos;
	break;
case 0x010B4:
	folded[outpos] = 0x02D14;
	++outpos;
	break;
case 0x010B5:
	folded[outpos] = 0x02D15;
	++outpos;
	break;
case 0x010B6:
	folded[outpos] = 0x02D16;
	++outpos;
	break;
case 0x010B7:
	folded[outpos] = 0x02D17;
	++outpos;
	break;
case 0x010B8:
	folded[outpos] = 0x02D18;
	++outpos;
	break;
case 0x010B9:
	folded[outpos] = 0x02D19;
	++outpos;
	break;
case 0x010BA:
	folded[outpos] = 0x02D1A;
	++outpos;
	break;
case 0x010BB:
	folded[outpos] = 0x02D1B;
	++outpos;
	break;
case 0x010BC:
	folded[outpos] = 0x02D1C;
	++outpos;
	break;
case 0x010BD:
	folded[outpos] = 0x02D1D;
	++outpos;
	break;
case 0x010BE:
	folded[outpos] = 0x02D1E;
	++outpos;
	break;
case 0x010BF:
	folded[outpos] = 0x02D1F;
	++outpos;
	break;
case 0x010C0:
	folded[outpos] = 0x02D20;
	++outpos;
	break;
case 0x010C1:
	folded[outpos] = 0x02D21;
	++outpos;
	break;
case 0x010C2:
	folded[outpos] = 0x02D22;
	++outpos;
	break;
case 0x010C3:
	folded[outpos] = 0x02D23;
	++outpos;
	break;
case 0x010C4:
	folded[outpos] = 0x02D24;
	++outpos;
	break;
case 0x010C5:
	folded[outpos] = 0x02D25;
	++outpos;
	break;
case 0x01E00:
	folded[outpos] = 0x01E01;
	++outpos;
	break;
case 0x01E02:
	folded[outpos] = 0x01E03;
	++outpos;
	break;
case 0x01E04:
	folded[outpos] = 0x01E05;
	++outpos;
	break;
case 0x01E06:
	folded[outpos] = 0x01E07;
	++outpos;
	break;
case 0x01E08:
	folded[outpos] = 0x01E09;
	++outpos;
	break;
case 0x01E0A:
	folded[outpos] = 0x01E0B;
	++outpos;
	break;
case 0x01E0C:
	folded[outpos] = 0x01E0D;
	++outpos;
	break;
case 0x01E0E:
	folded[outpos] = 0x01E0F;
	++outpos;
	break;
case 0x01E10:
	folded[outpos] = 0x01E11;
	++outpos;
	break;
case 0x01E12:
	folded[outpos] = 0x01E13;
	++outpos;
	break;
case 0x01E14:
	folded[outpos] = 0x01E15;
	++outpos;
	break;
case 0x01E16:
	folded[outpos] = 0x01E17;
	++outpos;
	break;
case 0x01E18:
	folded[outpos] = 0x01E19;
	++outpos;
	break;
case 0x01E1A:
	folded[outpos] = 0x01E1B;
	++outpos;
	break;
case 0x01E1C:
	folded[outpos] = 0x01E1D;
	++outpos;
	break;
case 0x01E1E:
	folded[outpos] = 0x01E1F;
	++outpos;
	break;
case 0x01E20:
	folded[outpos] = 0x01E21;
	++outpos;
	break;
case 0x01E22:
	folded[outpos] = 0x01E23;
	++outpos;
	break;
case 0x01E24:
	folded[outpos] = 0x01E25;
	++outpos;
	break;
case 0x01E26:
	folded[outpos] = 0x01E27;
	++outpos;
	break;
case 0x01E28:
	folded[outpos] = 0x01E29;
	++outpos;
	break;
case 0x01E2A:
	folded[outpos] = 0x01E2B;
	++outpos;
	break;
case 0x01E2C:
	folded[outpos] = 0x01E2D;
	++outpos;
	break;
case 0x01E2E:
	folded[outpos] = 0x01E2F;
	++outpos;
	break;
case 0x01E30:
	folded[outpos] = 0x01E31;
	++outpos;
	break;
case 0x01E32:
	folded[outpos] = 0x01E33;
	++outpos;
	break;
case 0x01E34:
	folded[outpos] = 0x01E35;
	++outpos;
	break;
case 0x01E36:
	folded[outpos] = 0x01E37;
	++outpos;
	break;
case 0x01E38:
	folded[outpos] = 0x01E39;
	++outpos;
	break;
case 0x01E3A:
	folded[outpos] = 0x01E3B;
	++outpos;
	break;
case 0x01E3C:
	folded[outpos] = 0x01E3D;
	++outpos;
	break;
case 0x01E3E:
	folded[outpos] = 0x01E3F;
	++outpos;
	break;
case 0x01E40:
	folded[outpos] = 0x01E41;
	++outpos;
	break;
case 0x01E42:
	folded[outpos] = 0x01E43;
	++outpos;
	break;
case 0x01E44:
	folded[outpos] = 0x01E45;
	++outpos;
	break;
case 0x01E46:
	folded[outpos] = 0x01E47;
	++outpos;
	break;
case 0x01E48:
	folded[outpos] = 0x01E49;
	++outpos;
	break;
case 0x01E4A:
	folded[outpos] = 0x01E4B;
	++outpos;
	break;
case 0x01E4C:
	folded[outpos] = 0x01E4D;
	++outpos;
	break;
case 0x01E4E:
	folded[outpos] = 0x01E4F;
	++outpos;
	break;
case 0x01E50:
	folded[outpos] = 0x01E51;
	++outpos;
	break;
case 0x01E52:
	folded[outpos] = 0x01E53;
	++outpos;
	break;
case 0x01E54:
	folded[outpos] = 0x01E55;
	++outpos;
	break;
case 0x01E56:
	folded[outpos] = 0x01E57;
	++outpos;
	break;
case 0x01E58:
	folded[outpos] = 0x01E59;
	++outpos;
	break;
case 0x01E5A:
	folded[outpos] = 0x01E5B;
	++outpos;
	break;
case 0x01E5C:
	folded[outpos] = 0x01E5D;
	++outpos;
	break;
case 0x01E5E:
	folded[outpos] = 0x01E5F;
	++outpos;
	break;
case 0x01E60:
	folded[outpos] = 0x01E61;
	++outpos;
	break;
case 0x01E62:
	folded[outpos] = 0x01E63;
	++outpos;
	break;
case 0x01E64:
	folded[outpos] = 0x01E65;
	++outpos;
	break;
case 0x01E66:
	folded[outpos] = 0x01E67;
	++outpos;
	break;
case 0x01E68:
	folded[outpos] = 0x01E69;
	++outpos;
	break;
case 0x01E6A:
	folded[outpos] = 0x01E6B;
	++outpos;
	break;
case 0x01E6C:
	folded[outpos] = 0x01E6D;
	++outpos;
	break;
case 0x01E6E:
	folded[outpos] = 0x01E6F;
	++outpos;
	break;
case 0x01E70:
	folded[outpos] = 0x01E71;
	++outpos;
	break;
case 0x01E72:
	folded[outpos] = 0x01E73;
	++outpos;
	break;
case 0x01E74:
	folded[outpos] = 0x01E75;
	++outpos;
	break;
case 0x01E76:
	folded[outpos] = 0x01E77;
	++outpos;
	break;
case 0x01E78:
	folded[outpos] = 0x01E79;
	++outpos;
	break;
case 0x01E7A:
	folded[outpos] = 0x01E7B;
	++outpos;
	break;
case 0x01E7C:
	folded[outpos] = 0x01E7D;
	++outpos;
	break;
case 0x01E7E:
	folded[outpos] = 0x01E7F;
	++outpos;
	break;
case 0x01E80:
	folded[outpos] = 0x01E81;
	++outpos;
	break;
case 0x01E82:
	folded[outpos] = 0x01E83;
	++outpos;
	break;
case 0x01E84:
	folded[outpos] = 0x01E85;
	++outpos;
	break;
case 0x01E86:
	folded[outpos] = 0x01E87;
	++outpos;
	break;
case 0x01E88:
	folded[outpos] = 0x01E89;
	++outpos;
	break;
case 0x01E8A:
	folded[outpos] = 0x01E8B;
	++outpos;
	break;
case 0x01E8C:
	folded[outpos] = 0x01E8D;
	++outpos;
	break;
case 0x01E8E:
	folded[outpos] = 0x01E8F;
	++outpos;
	break;
case 0x01E90:
	folded[outpos] = 0x01E91;
	++outpos;
	break;
case 0x01E92:
	folded[outpos] = 0x01E93;
	++outpos;
	break;
case 0x01E94:
	folded[outpos] = 0x01E95;
	++outpos;
	break;
case 0x01E96:
	folded[outpos] = 0x00068;
	++outpos;
	folded[outpos] = 0x00331;
	++outpos;
	break;
case 0x01E97:
	folded[outpos] = 0x00074;
	++outpos;
	folded[outpos] = 0x00308;
	++outpos;
	break;
case 0x01E98:
	folded[outpos] = 0x00077;
	++outpos;
	folded[outpos] = 0x0030A;
	++outpos;
	break;
case 0x01E99:
	folded[outpos] = 0x00079;
	++outpos;
	folded[outpos] = 0x0030A;
	++outpos;
	break;
case 0x01E9A:
	folded[outpos] = 0x00061;
	++outpos;
	folded[outpos] = 0x002BE;
	++outpos;
	break;
case 0x01E9B:
	folded[outpos] = 0x01E61;
	++outpos;
	break;
case 0x01E9E:
	folded[outpos] = 0x00073;
	++outpos;
	folded[outpos] = 0x00073;
	++outpos;
	break;
case 0x01EA0:
	folded[outpos] = 0x01EA1;
	++outpos;
	break;
case 0x01EA2:
	folded[outpos] = 0x01EA3;
	++outpos;
	break;
case 0x01EA4:
	folded[outpos] = 0x01EA5;
	++outpos;
	break;
case 0x01EA6:
	folded[outpos] = 0x01EA7;
	++outpos;
	break;
case 0x01EA8:
	folded[outpos] = 0x01EA9;
	++outpos;
	break;
case 0x01EAA:
	folded[outpos] = 0x01EAB;
	++outpos;
	break;
case 0x01EAC:
	folded[outpos] = 0x01EAD;
	++outpos;
	break;
case 0x01EAE:
	folded[outpos] = 0x01EAF;
	++outpos;
	break;
case 0x01EB0:
	folded[outpos] = 0x01EB1;
	++outpos;
	break;
case 0x01EB2:
	folded[outpos] = 0x01EB3;
	++outpos;
	break;
case 0x01EB4:
	folded[outpos] = 0x01EB5;
	++outpos;
	break;
case 0x01EB6:
	folded[outpos] = 0x01EB7;
	++outpos;
	break;
case 0x01EB8:
	folded[outpos] = 0x01EB9;
	++outpos;
	break;
case 0x01EBA:
	folded[outpos] = 0x01EBB;
	++outpos;
	break;
case 0x01EBC:
	folded[outpos] = 0x01EBD;
	++outpos;
	break;
case 0x01EBE:
	folded[outpos] = 0x01EBF;
	++outpos;
	break;
case 0x01EC0:
	folded[outpos] = 0x01EC1;
	++outpos;
	break;
case 0x01EC2:
	folded[outpos] = 0x01EC3;
	++outpos;
	break;
case 0x01EC4:
	folded[outpos] = 0x01EC5;
	++outpos;
	break;
case 0x01EC6:
	folded[outpos] = 0x01EC7;
	++outpos;
	break;
case 0x01EC8:
	folded[outpos] = 0x01EC9;
	++outpos;
	break;
case 0x01ECA:
	folded[outpos] = 0x01ECB;
	++outpos;
	break;
case 0x01ECC:
	folded[outpos] = 0x01ECD;
	++outpos;
	break;
case 0x01ECE:
	folded[outpos] = 0x01ECF;
	++outpos;
	break;
case 0x01ED0:
	folded[outpos] = 0x01ED1;
	++outpos;
	break;
case 0x01ED2:
	folded[outpos] = 0x01ED3;
	++outpos;
	break;
case 0x01ED4:
	folded[outpos] = 0x01ED5;
	++outpos;
	break;
case 0x01ED6:
	folded[outpos] = 0x01ED7;
	++outpos;
	break;
case 0x01ED8:
	folded[outpos] = 0x01ED9;
	++outpos;
	break;
case 0x01EDA:
	folded[outpos] = 0x01EDB;
	++outpos;
	break;
case 0x01EDC:
	folded[outpos] = 0x01EDD;
	++outpos;
	break;
case 0x01EDE:
	folded[outpos] = 0x01EDF;
	++outpos;
	break;
case 0x01EE0:
	folded[outpos] = 0x01EE1;
	++outpos;
	break;
case 0x01EE2:
	folded[outpos] = 0x01EE3;
	++outpos;
	break;
case 0x01EE4:
	folded[outpos] = 0x01EE5;
	++outpos;
	break;
case 0x01EE6:
	folded[outpos] = 0x01EE7;
	++outpos;
	break;
case 0x01EE8:
	folded[outpos] = 0x01EE9;
	++outpos;
	break;
case 0x01EEA:
	folded[outpos] = 0x01EEB;
	++outpos;
	break;
case 0x01EEC:
	folded[outpos] = 0x01EED;
	++outpos;
	break;
case 0x01EEE:
	folded[outpos] = 0x01EEF;
	++outpos;
	break;
case 0x01EF0:
	folded[outpos] = 0x01EF1;
	++outpos;
	break;
case 0x01EF2:
	folded[outpos] = 0x01EF3;
	++outpos;
	break;
case 0x01EF4:
	folded[outpos] = 0x01EF5;
	++outpos;
	break;
case 0x01EF6:
	folded[outpos] = 0x01EF7;
	++outpos;
	break;
case 0x01EF8:
	folded[outpos] = 0x01EF9;
	++outpos;
	break;
case 0x01EFA:
	folded[outpos] = 0x01EFB;
	++outpos;
	break;
case 0x01EFC:
	folded[outpos] = 0x01EFD;
	++outpos;
	break;
case 0x01EFE:
	folded[outpos] = 0x01EFF;
	++outpos;
	break;
case 0x01F08:
	folded[outpos] = 0x01F00;
	++outpos;
	break;
case 0x01F09:
	folded[outpos] = 0x01F01;
	++outpos;
	break;
case 0x01F0A:
	folded[outpos] = 0x01F02;
	++outpos;
	break;
case 0x01F0B:
	folded[outpos] = 0x01F03;
	++outpos;
	break;
case 0x01F0C:
	folded[outpos] = 0x01F04;
	++outpos;
	break;
case 0x01F0D:
	folded[outpos] = 0x01F05;
	++outpos;
	break;
case 0x01F0E:
	folded[outpos] = 0x01F06;
	++outpos;
	break;
case 0x01F0F:
	folded[outpos] = 0x01F07;
	++outpos;
	break;
case 0x01F18:
	folded[outpos] = 0x01F10;
	++outpos;
	break;
case 0x01F19:
	folded[outpos] = 0x01F11;
	++outpos;
	break;
case 0x01F1A:
	folded[outpos] = 0x01F12;
	++outpos;
	break;
case 0x01F1B:
	folded[outpos] = 0x01F13;
	++outpos;
	break;
case 0x01F1C:
	folded[outpos] = 0x01F14;
	++outpos;
	break;
case 0x01F1D:
	folded[outpos] = 0x01F15;
	++outpos;
	break;
case 0x01F28:
	folded[outpos] = 0x01F20;
	++outpos;
	break;
case 0x01F29:
	folded[outpos] = 0x01F21;
	++outpos;
	break;
case 0x01F2A:
	folded[outpos] = 0x01F22;
	++outpos;
	break;
case 0x01F2B:
	folded[outpos] = 0x01F23;
	++outpos;
	break;
case 0x01F2C:
	folded[outpos] = 0x01F24;
	++outpos;
	break;
case 0x01F2D:
	folded[outpos] = 0x01F25;
	++outpos;
	break;
case 0x01F2E:
	folded[outpos] = 0x01F26;
	++outpos;
	break;
case 0x01F2F:
	folded[outpos] = 0x01F27;
	++outpos;
	break;
case 0x01F38:
	folded[outpos] = 0x01F30;
	++outpos;
	break;
case 0x01F39:
	folded[outpos] = 0x01F31;
	++outpos;
	break;
case 0x01F3A:
	folded[outpos] = 0x01F32;
	++outpos;
	break;
case 0x01F3B:
	folded[outpos] = 0x01F33;
	++outpos;
	break;
case 0x01F3C:
	folded[outpos] = 0x01F34;
	++outpos;
	break;
case 0x01F3D:
	folded[outpos] = 0x01F35;
	++outpos;
	break;
case 0x01F3E:
	folded[outpos] = 0x01F36;
	++outpos;
	break;
case 0x01F3F:
	folded[outpos] = 0x01F37;
	++outpos;
	break;
case 0x01F48:
	folded[outpos] = 0x01F40;
	++outpos;
	break;
case 0x01F49:
	folded[outpos] = 0x01F41;
	++outpos;
	break;
case 0x01F4A:
	folded[outpos] = 0x01F42;
	++outpos;
	break;
case 0x01F4B:
	folded[outpos] = 0x01F43;
	++outpos;
	break;
case 0x01F4C:
	folded[outpos] = 0x01F44;
	++outpos;
	break;
case 0x01F4D:
	folded[outpos] = 0x01F45;
	++outpos;
	break;
case 0x01F50:
	folded[outpos] = 0x003C5;
	++outpos;
	folded[outpos] = 0x00313;
	++outpos;
	break;
case 0x01F52:
	folded[outpos] = 0x003C5;
	++outpos;
	folded[outpos] = 0x00313;
	++outpos;
	folded[outpos] = 0x00300;
	++outpos;
	break;
case 0x01F54:
	folded[outpos] = 0x003C5;
	++outpos;
	folded[outpos] = 0x00313;
	++outpos;
	folded[outpos] = 0x00301;
	++outpos;
	break;
case 0x01F56:
	folded[outpos] = 0x003C5;
	++outpos;
	folded[outpos] = 0x00313;
	++outpos;
	folded[outpos] = 0x00342;
	++outpos;
	break;
case 0x01F59:
	folded[outpos] = 0x01F51;
	++outpos;
	break;
case 0x01F5B:
	folded[outpos] = 0x01F53;
	++outpos;
	break;
case 0x01F5D:
	folded[outpos] = 0x01F55;
	++outpos;
	break;
case 0x01F5F:
	folded[outpos] = 0x01F57;
	++outpos;
	break;
case 0x01F68:
	folded[outpos] = 0x01F60;
	++outpos;
	break;
case 0x01F69:
	folded[outpos] = 0x01F61;
	++outpos;
	break;
case 0x01F6A:
	folded[outpos] = 0x01F62;
	++outpos;
	break;
case 0x01F6B:
	folded[outpos] = 0x01F63;
	++outpos;
	break;
case 0x01F6C:
	folded[outpos] = 0x01F64;
	++outpos;
	break;
case 0x01F6D:
	folded[outpos] = 0x01F65;
	++outpos;
	break;
case 0x01F6E:
	folded[outpos] = 0x01F66;
	++outpos;
	break;
case 0x01F6F:
	folded[outpos] = 0x01F67;
	++outpos;
	break;
case 0x01F80:
	folded[outpos] = 0x01F00;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F81:
	folded[outpos] = 0x01F01;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F82:
	folded[outpos] = 0x01F02;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F83:
	folded[outpos] = 0x01F03;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F84:
	folded[outpos] = 0x01F04;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F85:
	folded[outpos] = 0x01F05;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F86:
	folded[outpos] = 0x01F06;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F87:
	folded[outpos] = 0x01F07;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F88:
	folded[outpos] = 0x01F00;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F89:
	folded[outpos] = 0x01F01;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F8A:
	folded[outpos] = 0x01F02;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F8B:
	folded[outpos] = 0x01F03;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F8C:
	folded[outpos] = 0x01F04;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F8D:
	folded[outpos] = 0x01F05;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F8E:
	folded[outpos] = 0x01F06;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F8F:
	folded[outpos] = 0x01F07;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F90:
	folded[outpos] = 0x01F20;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F91:
	folded[outpos] = 0x01F21;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F92:
	folded[outpos] = 0x01F22;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F93:
	folded[outpos] = 0x01F23;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F94:
	folded[outpos] = 0x01F24;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F95:
	folded[outpos] = 0x01F25;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F96:
	folded[outpos] = 0x01F26;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F97:
	folded[outpos] = 0x01F27;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F98:
	folded[outpos] = 0x01F20;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F99:
	folded[outpos] = 0x01F21;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F9A:
	folded[outpos] = 0x01F22;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F9B:
	folded[outpos] = 0x01F23;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F9C:
	folded[outpos] = 0x01F24;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F9D:
	folded[outpos] = 0x01F25;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F9E:
	folded[outpos] = 0x01F26;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01F9F:
	folded[outpos] = 0x01F27;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FA0:
	folded[outpos] = 0x01F60;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FA1:
	folded[outpos] = 0x01F61;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FA2:
	folded[outpos] = 0x01F62;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FA3:
	folded[outpos] = 0x01F63;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FA4:
	folded[outpos] = 0x01F64;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FA5:
	folded[outpos] = 0x01F65;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FA6:
	folded[outpos] = 0x01F66;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FA7:
	folded[outpos] = 0x01F67;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FA8:
	folded[outpos] = 0x01F60;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FA9:
	folded[outpos] = 0x01F61;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FAA:
	folded[outpos] = 0x01F62;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FAB:
	folded[outpos] = 0x01F63;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FAC:
	folded[outpos] = 0x01F64;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FAD:
	folded[outpos] = 0x01F65;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FAE:
	folded[outpos] = 0x01F66;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FAF:
	folded[outpos] = 0x01F67;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FB2:
	folded[outpos] = 0x01F70;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FB3:
	folded[outpos] = 0x003B1;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FB4:
	folded[outpos] = 0x003AC;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FB6:
	folded[outpos] = 0x003B1;
	++outpos;
	folded[outpos] = 0x00342;
	++outpos;
	break;
case 0x01FB7:
	folded[outpos] = 0x003B1;
	++outpos;
	folded[outpos] = 0x00342;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FB8:
	folded[outpos] = 0x01FB0;
	++outpos;
	break;
case 0x01FB9:
	folded[outpos] = 0x01FB1;
	++outpos;
	break;
case 0x01FBA:
	folded[outpos] = 0x01F70;
	++outpos;
	break;
case 0x01FBB:
	folded[outpos] = 0x01F71;
	++outpos;
	break;
case 0x01FBC:
	folded[outpos] = 0x003B1;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FBE:
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FC2:
	folded[outpos] = 0x01F74;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FC3:
	folded[outpos] = 0x003B7;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FC4:
	folded[outpos] = 0x003AE;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FC6:
	folded[outpos] = 0x003B7;
	++outpos;
	folded[outpos] = 0x00342;
	++outpos;
	break;
case 0x01FC7:
	folded[outpos] = 0x003B7;
	++outpos;
	folded[outpos] = 0x00342;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FC8:
	folded[outpos] = 0x01F72;
	++outpos;
	break;
case 0x01FC9:
	folded[outpos] = 0x01F73;
	++outpos;
	break;
case 0x01FCA:
	folded[outpos] = 0x01F74;
	++outpos;
	break;
case 0x01FCB:
	folded[outpos] = 0x01F75;
	++outpos;
	break;
case 0x01FCC:
	folded[outpos] = 0x003B7;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FD2:
	folded[outpos] = 0x003B9;
	++outpos;
	folded[outpos] = 0x00308;
	++outpos;
	folded[outpos] = 0x00300;
	++outpos;
	break;
case 0x01FD3:
	folded[outpos] = 0x003B9;
	++outpos;
	folded[outpos] = 0x00308;
	++outpos;
	folded[outpos] = 0x00301;
	++outpos;
	break;
case 0x01FD6:
	folded[outpos] = 0x003B9;
	++outpos;
	folded[outpos] = 0x00342;
	++outpos;
	break;
case 0x01FD7:
	folded[outpos] = 0x003B9;
	++outpos;
	folded[outpos] = 0x00308;
	++outpos;
	folded[outpos] = 0x00342;
	++outpos;
	break;
case 0x01FD8:
	folded[outpos] = 0x01FD0;
	++outpos;
	break;
case 0x01FD9:
	folded[outpos] = 0x01FD1;
	++outpos;
	break;
case 0x01FDA:
	folded[outpos] = 0x01F76;
	++outpos;
	break;
case 0x01FDB:
	folded[outpos] = 0x01F77;
	++outpos;
	break;
case 0x01FE2:
	folded[outpos] = 0x003C5;
	++outpos;
	folded[outpos] = 0x00308;
	++outpos;
	folded[outpos] = 0x00300;
	++outpos;
	break;
case 0x01FE3:
	folded[outpos] = 0x003C5;
	++outpos;
	folded[outpos] = 0x00308;
	++outpos;
	folded[outpos] = 0x00301;
	++outpos;
	break;
case 0x01FE4:
	folded[outpos] = 0x003C1;
	++outpos;
	folded[outpos] = 0x00313;
	++outpos;
	break;
case 0x01FE6:
	folded[outpos] = 0x003C5;
	++outpos;
	folded[outpos] = 0x00342;
	++outpos;
	break;
case 0x01FE7:
	folded[outpos] = 0x003C5;
	++outpos;
	folded[outpos] = 0x00308;
	++outpos;
	folded[outpos] = 0x00342;
	++outpos;
	break;
case 0x01FE8:
	folded[outpos] = 0x01FE0;
	++outpos;
	break;
case 0x01FE9:
	folded[outpos] = 0x01FE1;
	++outpos;
	break;
case 0x01FEA:
	folded[outpos] = 0x01F7A;
	++outpos;
	break;
case 0x01FEB:
	folded[outpos] = 0x01F7B;
	++outpos;
	break;
case 0x01FEC:
	folded[outpos] = 0x01FE5;
	++outpos;
	break;
case 0x01FF2:
	folded[outpos] = 0x01F7C;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FF3:
	folded[outpos] = 0x003C9;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FF4:
	folded[outpos] = 0x003CE;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FF6:
	folded[outpos] = 0x003C9;
	++outpos;
	folded[outpos] = 0x00342;
	++outpos;
	break;
case 0x01FF7:
	folded[outpos] = 0x003C9;
	++outpos;
	folded[outpos] = 0x00342;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x01FF8:
	folded[outpos] = 0x01F78;
	++outpos;
	break;
case 0x01FF9:
	folded[outpos] = 0x01F79;
	++outpos;
	break;
case 0x01FFA:
	folded[outpos] = 0x01F7C;
	++outpos;
	break;
case 0x01FFB:
	folded[outpos] = 0x01F7D;
	++outpos;
	break;
case 0x01FFC:
	folded[outpos] = 0x003C9;
	++outpos;
	folded[outpos] = 0x003B9;
	++outpos;
	break;
case 0x02126:
	folded[outpos] = 0x003C9;
	++outpos;
	break;
case 0x0212A:
	folded[outpos] = 0x0006B;
	++outpos;
	break;
case 0x0212B:
	folded[outpos] = 0x000E5;
	++outpos;
	break;
case 0x02132:
	folded[outpos] = 0x0214E;
	++outpos;
	break;
case 0x02160:
	folded[outpos] = 0x02170;
	++outpos;
	break;
case 0x02161:
	folded[outpos] = 0x02171;
	++outpos;
	break;
case 0x02162:
	folded[outpos] = 0x02172;
	++outpos;
	break;
case 0x02163:
	folded[outpos] = 0x02173;
	++outpos;
	break;
case 0x02164:
	folded[outpos] = 0x02174;
	++outpos;
	break;
case 0x02165:
	folded[outpos] = 0x02175;
	++outpos;
	break;
case 0x02166:
	folded[outpos] = 0x02176;
	++outpos;
	break;
case 0x02167:
	folded[outpos] = 0x02177;
	++outpos;
	break;
case 0x02168:
	folded[outpos] = 0x02178;
	++outpos;
	break;
case 0x02169:
	folded[outpos] = 0x02179;
	++outpos;
	break;
case 0x0216A:
	folded[outpos] = 0x0217A;
	++outpos;
	break;
case 0x0216B:
	folded[outpos] = 0x0217B;
	++outpos;
	break;
case 0x0216C:
	folded[outpos] = 0x0217C;
	++outpos;
	break;
case 0x0216D:
	folded[outpos] = 0x0217D;
	++outpos;
	break;
case 0x0216E:
	folded[outpos] = 0x0217E;
	++outpos;
	break;
case 0x0216F:
	folded[outpos] = 0x0217F;
	++outpos;
	break;
case 0x02183:
	folded[outpos] = 0x02184;
	++outpos;
	break;
case 0x024B6:
	folded[outpos] = 0x024D0;
	++outpos;
	break;
case 0x024B7:
	folded[outpos] = 0x024D1;
	++outpos;
	break;
case 0x024B8:
	folded[outpos] = 0x024D2;
	++outpos;
	break;
case 0x024B9:
	folded[outpos] = 0x024D3;
	++outpos;
	break;
case 0x024BA:
	folded[outpos] = 0x024D4;
	++outpos;
	break;
case 0x024BB:
	folded[outpos] = 0x024D5;
	++outpos;
	break;
case 0x024BC:
	folded[outpos] = 0x024D6;
	++outpos;
	break;
case 0x024BD:
	folded[outpos] = 0x024D7;
	++outpos;
	break;
case 0x024BE:
	folded[outpos] = 0x024D8;
	++outpos;
	break;
case 0x024BF:
	folded[outpos] = 0x024D9;
	++outpos;
	break;
case 0x024C0:
	folded[outpos] = 0x024DA;
	++outpos;
	break;
case 0x024C1:
	folded[outpos] = 0x024DB;
	++outpos;
	break;
case 0x024C2:
	folded[outpos] = 0x024DC;
	++outpos;
	break;
case 0x024C3:
	folded[outpos] = 0x024DD;
	++outpos;
	break;
case 0x024C4:
	folded[outpos] = 0x024DE;
	++outpos;
	break;
case 0x024C5:
	folded[outpos] = 0x024DF;
	++outpos;
	break;
case 0x024C6:
	folded[outpos] = 0x024E0;
	++outpos;
	break;
case 0x024C7:
	folded[outpos] = 0x024E1;
	++outpos;
	break;
case 0x024C8:
	folded[outpos] = 0x024E2;
	++outpos;
	break;
case 0x024C9:
	folded[outpos] = 0x024E3;
	++outpos;
	break;
case 0x024CA:
	folded[outpos] = 0x024E4;
	++outpos;
	break;
case 0x024CB:
	folded[outpos] = 0x024E5;
	++outpos;
	break;
case 0x024CC:
	folded[outpos] = 0x024E6;
	++outpos;
	break;
case 0x024CD:
	folded[outpos] = 0x024E7;
	++outpos;
	break;
case 0x024CE:
	folded[outpos] = 0x024E8;
	++outpos;
	break;
case 0x024CF:
	folded[outpos] = 0x024E9;
	++outpos;
	break;
case 0x02C00:
	folded[outpos] = 0x02C30;
	++outpos;
	break;
case 0x02C01:
	folded[outpos] = 0x02C31;
	++outpos;
	break;
case 0x02C02:
	folded[outpos] = 0x02C32;
	++outpos;
	break;
case 0x02C03:
	folded[outpos] = 0x02C33;
	++outpos;
	break;
case 0x02C04:
	folded[outpos] = 0x02C34;
	++outpos;
	break;
case 0x02C05:
	folded[outpos] = 0x02C35;
	++outpos;
	break;
case 0x02C06:
	folded[outpos] = 0x02C36;
	++outpos;
	break;
case 0x02C07:
	folded[outpos] = 0x02C37;
	++outpos;
	break;
case 0x02C08:
	folded[outpos] = 0x02C38;
	++outpos;
	break;
case 0x02C09:
	folded[outpos] = 0x02C39;
	++outpos;
	break;
case 0x02C0A:
	folded[outpos] = 0x02C3A;
	++outpos;
	break;
case 0x02C0B:
	folded[outpos] = 0x02C3B;
	++outpos;
	break;
case 0x02C0C:
	folded[outpos] = 0x02C3C;
	++outpos;
	break;
case 0x02C0D:
	folded[outpos] = 0x02C3D;
	++outpos;
	break;
case 0x02C0E:
	folded[outpos] = 0x02C3E;
	++outpos;
	break;
case 0x02C0F:
	folded[outpos] = 0x02C3F;
	++outpos;
	break;
case 0x02C10:
	folded[outpos] = 0x02C40;
	++outpos;
	break;
case 0x02C11:
	folded[outpos] = 0x02C41;
	++outpos;
	break;
case 0x02C12:
	folded[outpos] = 0x02C42;
	++outpos;
	break;
case 0x02C13:
	folded[outpos] = 0x02C43;
	++outpos;
	break;
case 0x02C14:
	folded[outpos] = 0x02C44;
	++outpos;
	break;
case 0x02C15:
	folded[outpos] = 0x02C45;
	++outpos;
	break;
case 0x02C16:
	folded[outpos] = 0x02C46;
	++outpos;
	break;
case 0x02C17:
	folded[outpos] = 0x02C47;
	++outpos;
	break;
case 0x02C18:
	folded[outpos] = 0x02C48;
	++outpos;
	break;
case 0x02C19:
	folded[outpos] = 0x02C49;
	++outpos;
	break;
case 0x02C1A:
	folded[outpos] = 0x02C4A;
	++outpos;
	break;
case 0x02C1B:
	folded[outpos] = 0x02C4B;
	++outpos;
	break;
case 0x02C1C:
	folded[outpos] = 0x02C4C;
	++outpos;
	break;
case 0x02C1D:
	folded[outpos] = 0x02C4D;
	++outpos;
	break;
case 0x02C1E:
	folded[outpos] = 0x02C4E;
	++outpos;
	break;
case 0x02C1F:
	folded[outpos] = 0x02C4F;
	++outpos;
	break;
case 0x02C20:
	folded[outpos] = 0x02C50;
	++outpos;
	break;
case 0x02C21:
	folded[outpos] = 0x02C51;
	++outpos;
	break;
case 0x02C22:
	folded[outpos] = 0x02C52;
	++outpos;
	break;
case 0x02C23:
	folded[outpos] = 0x02C53;
	++outpos;
	break;
case 0x02C24:
	folded[outpos] = 0x02C54;
	++outpos;
	break;
case 0x02C25:
	folded[outpos] = 0x02C55;
	++outpos;
	break;
case 0x02C26:
	folded[outpos] = 0x02C56;
	++outpos;
	break;
case 0x02C27:
	folded[outpos] = 0x02C57;
	++outpos;
	break;
case 0x02C28:
	folded[outpos] = 0x02C58;
	++outpos;
	break;
case 0x02C29:
	folded[outpos] = 0x02C59;
	++outpos;
	break;
case 0x02C2A:
	folded[outpos] = 0x02C5A;
	++outpos;
	break;
case 0x02C2B:
	folded[outpos] = 0x02C5B;
	++outpos;
	break;
case 0x02C2C:
	folded[outpos] = 0x02C5C;
	++outpos;
	break;
case 0x02C2D:
	folded[outpos] = 0x02C5D;
	++outpos;
	break;
case 0x02C2E:
	folded[outpos] = 0x02C5E;
	++outpos;
	break;
case 0x02C60:
	folded[outpos] = 0x02C61;
	++outpos;
	break;
case 0x02C62:
	folded[outpos] = 0x0026B;
	++outpos;
	break;
case 0x02C63:
	folded[outpos] = 0x01D7D;
	++outpos;
	break;
case 0x02C64:
	folded[outpos] = 0x0027D;
	++outpos;
	break;
case 0x02C67:
	folded[outpos] = 0x02C68;
	++outpos;
	break;
case 0x02C69:
	folded[outpos] = 0x02C6A;
	++outpos;
	break;
case 0x02C6B:
	folded[outpos] = 0x02C6C;
	++outpos;
	break;
case 0x02C6D:
	folded[outpos] = 0x00251;
	++outpos;
	break;
case 0x02C6E:
	folded[outpos] = 0x00271;
	++outpos;
	break;
case 0x02C6F:
	folded[outpos] = 0x00250;
	++outpos;
	break;
case 0x02C72:
	folded[outpos] = 0x02C73;
	++outpos;
	break;
case 0x02C75:
	folded[outpos] = 0x02C76;
	++outpos;
	break;
case 0x02C80:
	folded[outpos] = 0x02C81;
	++outpos;
	break;
case 0x02C82:
	folded[outpos] = 0x02C83;
	++outpos;
	break;
case 0x02C84:
	folded[outpos] = 0x02C85;
	++outpos;
	break;
case 0x02C86:
	folded[outpos] = 0x02C87;
	++outpos;
	break;
case 0x02C88:
	folded[outpos] = 0x02C89;
	++outpos;
	break;
case 0x02C8A:
	folded[outpos] = 0x02C8B;
	++outpos;
	break;
case 0x02C8C:
	folded[outpos] = 0x02C8D;
	++outpos;
	break;
case 0x02C8E:
	folded[outpos] = 0x02C8F;
	++outpos;
	break;
case 0x02C90:
	folded[outpos] = 0x02C91;
	++outpos;
	break;
case 0x02C92:
	folded[outpos] = 0x02C93;
	++outpos;
	break;
case 0x02C94:
	folded[outpos] = 0x02C95;
	++outpos;
	break;
case 0x02C96:
	folded[outpos] = 0x02C97;
	++outpos;
	break;
case 0x02C98:
	folded[outpos] = 0x02C99;
	++outpos;
	break;
case 0x02C9A:
	folded[outpos] = 0x02C9B;
	++outpos;
	break;
case 0x02C9C:
	folded[outpos] = 0x02C9D;
	++outpos;
	break;
case 0x02C9E:
	folded[outpos] = 0x02C9F;
	++outpos;
	break;
case 0x02CA0:
	folded[outpos] = 0x02CA1;
	++outpos;
	break;
case 0x02CA2:
	folded[outpos] = 0x02CA3;
	++outpos;
	break;
case 0x02CA4:
	folded[outpos] = 0x02CA5;
	++outpos;
	break;
case 0x02CA6:
	folded[outpos] = 0x02CA7;
	++outpos;
	break;
case 0x02CA8:
	folded[outpos] = 0x02CA9;
	++outpos;
	break;
case 0x02CAA:
	folded[outpos] = 0x02CAB;
	++outpos;
	break;
case 0x02CAC:
	folded[outpos] = 0x02CAD;
	++outpos;
	break;
case 0x02CAE:
	folded[outpos] = 0x02CAF;
	++outpos;
	break;
case 0x02CB0:
	folded[outpos] = 0x02CB1;
	++outpos;
	break;
case 0x02CB2:
	folded[outpos] = 0x02CB3;
	++outpos;
	break;
case 0x02CB4:
	folded[outpos] = 0x02CB5;
	++outpos;
	break;
case 0x02CB6:
	folded[outpos] = 0x02CB7;
	++outpos;
	break;
case 0x02CB8:
	folded[outpos] = 0x02CB9;
	++outpos;
	break;
case 0x02CBA:
	folded[outpos] = 0x02CBB;
	++outpos;
	break;
case 0x02CBC:
	folded[outpos] = 0x02CBD;
	++outpos;
	break;
case 0x02CBE:
	folded[outpos] = 0x02CBF;
	++outpos;
	break;
case 0x02CC0:
	folded[outpos] = 0x02CC1;
	++outpos;
	break;
case 0x02CC2:
	folded[outpos] = 0x02CC3;
	++outpos;
	break;
case 0x02CC4:
	folded[outpos] = 0x02CC5;
	++outpos;
	break;
case 0x02CC6:
	folded[outpos] = 0x02CC7;
	++outpos;
	break;
case 0x02CC8:
	folded[outpos] = 0x02CC9;
	++outpos;
	break;
case 0x02CCA:
	folded[outpos] = 0x02CCB;
	++outpos;
	break;
case 0x02CCC:
	folded[outpos] = 0x02CCD;
	++outpos;
	break;
case 0x02CCE:
	folded[outpos] = 0x02CCF;
	++outpos;
	break;
case 0x02CD0:
	folded[outpos] = 0x02CD1;
	++outpos;
	break;
case 0x02CD2:
	folded[outpos] = 0x02CD3;
	++outpos;
	break;
case 0x02CD4:
	folded[outpos] = 0x02CD5;
	++outpos;
	break;
case 0x02CD6:
	folded[outpos] = 0x02CD7;
	++outpos;
	break;
case 0x02CD8:
	folded[outpos] = 0x02CD9;
	++outpos;
	break;
case 0x02CDA:
	folded[outpos] = 0x02CDB;
	++outpos;
	break;
case 0x02CDC:
	folded[outpos] = 0x02CDD;
	++outpos;
	break;
case 0x02CDE:
	folded[outpos] = 0x02CDF;
	++outpos;
	break;
case 0x02CE0:
	folded[outpos] = 0x02CE1;
	++outpos;
	break;
case 0x02CE2:
	folded[outpos] = 0x02CE3;
	++outpos;
	break;
case 0x0A640:
	folded[outpos] = 0x0A641;
	++outpos;
	break;
case 0x0A642:
	folded[outpos] = 0x0A643;
	++outpos;
	break;
case 0x0A644:
	folded[outpos] = 0x0A645;
	++outpos;
	break;
case 0x0A646:
	folded[outpos] = 0x0A647;
	++outpos;
	break;
case 0x0A648:
	folded[outpos] = 0x0A649;
	++outpos;
	break;
case 0x0A64A:
	folded[outpos] = 0x0A64B;
	++outpos;
	break;
case 0x0A64C:
	folded[outpos] = 0x0A64D;
	++outpos;
	break;
case 0x0A64E:
	folded[outpos] = 0x0A64F;
	++outpos;
	break;
case 0x0A650:
	folded[outpos] = 0x0A651;
	++outpos;
	break;
case 0x0A652:
	folded[outpos] = 0x0A653;
	++outpos;
	break;
case 0x0A654:
	folded[outpos] = 0x0A655;
	++outpos;
	break;
case 0x0A656:
	folded[outpos] = 0x0A657;
	++outpos;
	break;
case 0x0A658:
	folded[outpos] = 0x0A659;
	++outpos;
	break;
case 0x0A65A:
	folded[outpos] = 0x0A65B;
	++outpos;
	break;
case 0x0A65C:
	folded[outpos] = 0x0A65D;
	++outpos;
	break;
case 0x0A65E:
	folded[outpos] = 0x0A65F;
	++outpos;
	break;
case 0x0A662:
	folded[outpos] = 0x0A663;
	++outpos;
	break;
case 0x0A664:
	folded[outpos] = 0x0A665;
	++outpos;
	break;
case 0x0A666:
	folded[outpos] = 0x0A667;
	++outpos;
	break;
case 0x0A668:
	folded[outpos] = 0x0A669;
	++outpos;
	break;
case 0x0A66A:
	folded[outpos] = 0x0A66B;
	++outpos;
	break;
case 0x0A66C:
	folded[outpos] = 0x0A66D;
	++outpos;
	break;
case 0x0A680:
	folded[outpos] = 0x0A681;
	++outpos;
	break;
case 0x0A682:
	folded[outpos] = 0x0A683;
	++outpos;
	break;
case 0x0A684:
	folded[outpos] = 0x0A685;
	++outpos;
	break;
case 0x0A686:
	folded[outpos] = 0x0A687;
	++outpos;
	break;
case 0x0A688:
	folded[outpos] = 0x0A689;
	++outpos;
	break;
case 0x0A68A:
	folded[outpos] = 0x0A68B;
	++outpos;
	break;
case 0x0A68C:
	folded[outpos] = 0x0A68D;
	++outpos;
	break;
case 0x0A68E:
	folded[outpos] = 0x0A68F;
	++outpos;
	break;
case 0x0A690:
	folded[outpos] = 0x0A691;
	++outpos;
	break;
case 0x0A692:
	folded[outpos] = 0x0A693;
	++outpos;
	break;
case 0x0A694:
	folded[outpos] = 0x0A695;
	++outpos;
	break;
case 0x0A696:
	folded[outpos] = 0x0A697;
	++outpos;
	break;
case 0x0A722:
	folded[outpos] = 0x0A723;
	++outpos;
	break;
case 0x0A724:
	folded[outpos] = 0x0A725;
	++outpos;
	break;
case 0x0A726:
	folded[outpos] = 0x0A727;
	++outpos;
	break;
case 0x0A728:
	folded[outpos] = 0x0A729;
	++outpos;
	break;
case 0x0A72A:
	folded[outpos] = 0x0A72B;
	++outpos;
	break;
case 0x0A72C:
	folded[outpos] = 0x0A72D;
	++outpos;
	break;
case 0x0A72E:
	folded[outpos] = 0x0A72F;
	++outpos;
	break;
case 0x0A732:
	folded[outpos] = 0x0A733;
	++outpos;
	break;
case 0x0A734:
	folded[outpos] = 0x0A735;
	++outpos;
	break;
case 0x0A736:
	folded[outpos] = 0x0A737;
	++outpos;
	break;
case 0x0A738:
	folded[outpos] = 0x0A739;
	++outpos;
	break;
case 0x0A73A:
	folded[outpos] = 0x0A73B;
	++outpos;
	break;
case 0x0A73C:
	folded[outpos] = 0x0A73D;
	++outpos;
	break;
case 0x0A73E:
	folded[outpos] = 0x0A73F;
	++outpos;
	break;
case 0x0A740:
	folded[outpos] = 0x0A741;
	++outpos;
	break;
case 0x0A742:
	folded[outpos] = 0x0A743;
	++outpos;
	break;
case 0x0A744:
	folded[outpos] = 0x0A745;
	++outpos;
	break;
case 0x0A746:
	folded[outpos] = 0x0A747;
	++outpos;
	break;
case 0x0A748:
	folded[outpos] = 0x0A749;
	++outpos;
	break;
case 0x0A74A:
	folded[outpos] = 0x0A74B;
	++outpos;
	break;
case 0x0A74C:
	folded[outpos] = 0x0A74D;
	++outpos;
	break;
case 0x0A74E:
	folded[outpos] = 0x0A74F;
	++outpos;
	break;
case 0x0A750:
	folded[outpos] = 0x0A751;
	++outpos;
	break;
case 0x0A752:
	folded[outpos] = 0x0A753;
	++outpos;
	break;
case 0x0A754:
	folded[outpos] = 0x0A755;
	++outpos;
	break;
case 0x0A756:
	folded[outpos] = 0x0A757;
	++outpos;
	break;
case 0x0A758:
	folded[outpos] = 0x0A759;
	++outpos;
	break;
case 0x0A75A:
	folded[outpos] = 0x0A75B;
	++outpos;
	break;
case 0x0A75C:
	folded[outpos] = 0x0A75D;
	++outpos;
	break;
case 0x0A75E:
	folded[outpos] = 0x0A75F;
	++outpos;
	break;
case 0x0A760:
	folded[outpos] = 0x0A761;
	++outpos;
	break;
case 0x0A762:
	folded[outpos] = 0x0A763;
	++outpos;
	break;
case 0x0A764:
	folded[outpos] = 0x0A765;
	++outpos;
	break;
case 0x0A766:
	folded[outpos] = 0x0A767;
	++outpos;
	break;
case 0x0A768:
	folded[outpos] = 0x0A769;
	++outpos;
	break;
case 0x0A76A:
	folded[outpos] = 0x0A76B;
	++outpos;
	break;
case 0x0A76C:
	folded[outpos] = 0x0A76D;
	++outpos;
	break;
case 0x0A76E:
	folded[outpos] = 0x0A76F;
	++outpos;
	break;
case 0x0A779:
	folded[outpos] = 0x0A77A;
	++outpos;
	break;
case 0x0A77B:
	folded[outpos] = 0x0A77C;
	++outpos;
	break;
case 0x0A77D:
	folded[outpos] = 0x01D79;
	++outpos;
	break;
case 0x0A77E:
	folded[outpos] = 0x0A77F;
	++outpos;
	break;
case 0x0A780:
	folded[outpos] = 0x0A781;
	++outpos;
	break;
case 0x0A782:
	folded[outpos] = 0x0A783;
	++outpos;
	break;
case 0x0A784:
	folded[outpos] = 0x0A785;
	++outpos;
	break;
case 0x0A786:
	folded[outpos] = 0x0A787;
	++outpos;
	break;
case 0x0A78B:
	folded[outpos] = 0x0A78C;
	++outpos;
	break;
case 0x0FB00:
	folded[outpos] = 0x00066;
	++outpos;
	folded[outpos] = 0x00066;
	++outpos;
	break;
case 0x0FB01:
	folded[outpos] = 0x00066;
	++outpos;
	folded[outpos] = 0x00069;
	++outpos;
	break;
case 0x0FB02:
	folded[outpos] = 0x00066;
	++outpos;
	folded[outpos] = 0x0006C;
	++outpos;
	break;
case 0x0FB03:
	folded[outpos] = 0x00066;
	++outpos;
	folded[outpos] = 0x00066;
	++outpos;
	folded[outpos] = 0x00069;
	++outpos;
	break;
case 0x0FB04:
	folded[outpos] = 0x00066;
	++outpos;
	folded[outpos] = 0x00066;
	++outpos;
	folded[outpos] = 0x0006C;
	++outpos;
	break;
case 0x0FB05:
	folded[outpos] = 0x00073;
	++outpos;
	folded[outpos] = 0x00074;
	++outpos;
	break;
case 0x0FB06:
	folded[outpos] = 0x00073;
	++outpos;
	folded[outpos] = 0x00074;
	++outpos;
	break;
case 0x0FB13:
	folded[outpos] = 0x00574;
	++outpos;
	folded[outpos] = 0x00576;
	++outpos;
	break;
case 0x0FB14:
	folded[outpos] = 0x00574;
	++outpos;
	folded[outpos] = 0x00565;
	++outpos;
	break;
case 0x0FB15:
	folded[outpos] = 0x00574;
	++outpos;
	folded[outpos] = 0x0056B;
	++outpos;
	break;
case 0x0FB16:
	folded[outpos] = 0x0057E;
	++outpos;
	folded[outpos] = 0x00576;
	++outpos;
	break;
case 0x0FB17:
	folded[outpos] = 0x00574;
	++outpos;
	folded[outpos] = 0x0056D;
	++outpos;
	break;
case 0x0FF21:
	folded[outpos] = 0x0FF41;
	++outpos;
	break;
case 0x0FF22:
	folded[outpos] = 0x0FF42;
	++outpos;
	break;
case 0x0FF23:
	folded[outpos] = 0x0FF43;
	++outpos;
	break;
case 0x0FF24:
	folded[outpos] = 0x0FF44;
	++outpos;
	break;
case 0x0FF25:
	folded[outpos] = 0x0FF45;
	++outpos;
	break;
case 0x0FF26:
	folded[outpos] = 0x0FF46;
	++outpos;
	break;
case 0x0FF27:
	folded[outpos] = 0x0FF47;
	++outpos;
	break;
case 0x0FF28:
	folded[outpos] = 0x0FF48;
	++outpos;
	break;
case 0x0FF29:
	folded[outpos] = 0x0FF49;
	++outpos;
	break;
case 0x0FF2A:
	folded[outpos] = 0x0FF4A;
	++outpos;
	break;
case 0x0FF2B:
	folded[outpos] = 0x0FF4B;
	++outpos;
	break;
case 0x0FF2C:
	folded[outpos] = 0x0FF4C;
	++outpos;
	break;
case 0x0FF2D:
	folded[outpos] = 0x0FF4D;
	++outpos;
	break;
case 0x0FF2E:
	folded[outpos] = 0x0FF4E;
	++outpos;
	break;
case 0x0FF2F:
	folded[outpos] = 0x0FF4F;
	++outpos;
	break;
case 0x0FF30:
	folded[outpos] = 0x0FF50;
	++outpos;
	break;
case 0x0FF31:
	folded[outpos] = 0x0FF51;
	++outpos;
	break;
case 0x0FF32:
	folded[outpos] = 0x0FF52;
	++outpos;
	break;
case 0x0FF33:
	folded[outpos] = 0x0FF53;
	++outpos;
	break;
case 0x0FF34:
	folded[outpos] = 0x0FF54;
	++outpos;
	break;
case 0x0FF35:
	folded[outpos] = 0x0FF55;
	++outpos;
	break;
case 0x0FF36:
	folded[outpos] = 0x0FF56;
	++outpos;
	break;
case 0x0FF37:
	folded[outpos] = 0x0FF57;
	++outpos;
	break;
case 0x0FF38:
	folded[outpos] = 0x0FF58;
	++outpos;
	break;
case 0x0FF39:
	folded[outpos] = 0x0FF59;
	++outpos;
	break;
case 0x0FF3A:
	folded[outpos] = 0x0FF5A;
	++outpos;
	break;
case 0x10400:
	folded[outpos] = 0x10428;
	++outpos;
	break;
case 0x10401:
	folded[outpos] = 0x10429;
	++outpos;
	break;
case 0x10402:
	folded[outpos] = 0x1042A;
	++outpos;
	break;
case 0x10403:
	folded[outpos] = 0x1042B;
	++outpos;
	break;
case 0x10404:
	folded[outpos] = 0x1042C;
	++outpos;
	break;
case 0x10405:
	folded[outpos] = 0x1042D;
	++outpos;
	break;
case 0x10406:
	folded[outpos] = 0x1042E;
	++outpos;
	break;
case 0x10407:
	folded[outpos] = 0x1042F;
	++outpos;
	break;
case 0x10408:
	folded[outpos] = 0x10430;
	++outpos;
	break;
case 0x10409:
	folded[outpos] = 0x10431;
	++outpos;
	break;
case 0x1040A:
	folded[outpos] = 0x10432;
	++outpos;
	break;
case 0x1040B:
	folded[outpos] = 0x10433;
	++outpos;
	break;
case 0x1040C:
	folded[outpos] = 0x10434;
	++outpos;
	break;
case 0x1040D:
	folded[outpos] = 0x10435;
	++outpos;
	break;
case 0x1040E:
	folded[outpos] = 0x10436;
	++outpos;
	break;
case 0x1040F:
	folded[outpos] = 0x10437;
	++outpos;
	break;
case 0x10410:
	folded[outpos] = 0x10438;
	++outpos;
	break;
case 0x10411:
	folded[outpos] = 0x10439;
	++outpos;
	break;
case 0x10412:
	folded[outpos] = 0x1043A;
	++outpos;
	break;
case 0x10413:
	folded[outpos] = 0x1043B;
	++outpos;
	break;
case 0x10414:
	folded[outpos] = 0x1043C;
	++outpos;
	break;
case 0x10415:
	folded[outpos] = 0x1043D;
	++outpos;
	break;
case 0x10416:
	folded[outpos] = 0x1043E;
	++outpos;
	break;
case 0x10417:
	folded[outpos] = 0x1043F;
	++outpos;
	break;
case 0x10418:
	folded[outpos] = 0x10440;
	++outpos;
	break;
case 0x10419:
	folded[outpos] = 0x10441;
	++outpos;
	break;
case 0x1041A:
	folded[outpos] = 0x10442;
	++outpos;
	break;
case 0x1041B:
	folded[outpos] = 0x10443;
	++outpos;
	break;
case 0x1041C:
	folded[outpos] = 0x10444;
	++outpos;
	break;
case 0x1041D:
	folded[outpos] = 0x10445;
	++outpos;
	break;
case 0x1041E:
	folded[outpos] = 0x10446;
	++outpos;
	break;
case 0x1041F:
	folded[outpos] = 0x10447;
	++outpos;
	break;
case 0x10420:
	folded[outpos] = 0x10448;
	++outpos;
	break;
case 0x10421:
	folded[outpos] = 0x10449;
	++outpos;
	break;
case 0x10422:
	folded[outpos] = 0x1044A;
	++outpos;
	break;
case 0x10423:
	folded[outpos] = 0x1044B;
	++outpos;
	break;
case 0x10424:
	folded[outpos] = 0x1044C;
	++outpos;
	break;
case 0x10425:
	folded[outpos] = 0x1044D;
	++outpos;
	break;
case 0x10426:
	folded[outpos] = 0x1044E;
	++outpos;
	break;
case 0x10427:
	folded[outpos] = 0x1044F;
	++outpos;
	break;
