/***************************************************************************
                          cencrypt.h  -  description
                             -------------------
    begin                : Sat Sep 29 2001
    copyright            : (C) 2001-2003 by Mathias Küster
    email                : mathen@users.berlios.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CENCRYPT_H
#define CENCRYPT_H

/**
  *@author Mathias Küster
  *
  * This class is just badly named, it does the lock to key
  * routine, including the %DCN???% escaping.
  */

#include <dclib/dcos.h>
#include <dclib/core/cstring.h>

class CEncrypt {
public: 
	/** */
	static void Encrypt( CString s, CString & d );
	/** */
	static void Encode( int c, CString & d );
	/** */
	static CString Encode( CString s );
	/** */
	static CString Decode( CString s );
	
protected:
	/** */
	CEncrypt();
	/** */
	virtual ~CEncrypt();
};

#endif
