/*
 * This files fixes that the hashing code from DC++
 * can be used almost verbatim.
 */

#ifndef _INCLUDED_COMPAT_H_
#define _INCLUDED_COMPAT_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef uint64_t
#include <stdint.h>
#endif

// standard includes
#include <string.h>
#include <vector>
#include <utility>
#include <algorithm>
using std::pair;
using std::make_pair;
using std::vector;
using std::min;
using std::max;

// algorithm include added for min and max functions
// also this typedef added for 0.707 from DCPlusPlus.h
typedef std::vector<uint8_t> ByteVector;

// defines used by the DC++ code
#ifndef _ULL
#if !defined(__MINGW32__) && defined(SIZEOF_LONG) && (SIZEOF_LONG == 8)
#define _ULL(x) x##ul
#else
#define _ULL(x) x##ull
#endif
#endif
#define dcassert(x) 


#endif //_INCLUDED_COMPAT_H_
